@@ -69,14 +69,14 @@ class ActionManagerImpl::NodeSelectVisitor : public TreeNodeVisitor
     public:
     NodeSelectVisitor(ActionManagerImpl* manager) : m_manager(manager) {}
 
-    virtual bool visitFeed(Feed* /*node*/)
+    virtual bool visitFeed(Feed* node)
     {
         KAction* remove = m_manager->action("feed_remove");
         if (remove)
             remove->setEnabled(true);
         KAction* hp = m_manager->action("feed_homepage");
         if (hp)
-            remove->setEnabled(true);
+            hp->setEnabled(!node->htmlUrl().isEmpty());
         m_manager->action("feed_fetch")->setText(i18n("&Fetch Feed"));
         m_manager->action("feed_remove")->setText(i18n("&Delete Feed"));
         m_manager->action("feed_modify")->setText(i18n("&Edit Feed..."));
@@ -184,9 +184,12 @@ void ActionManagerImpl::setTagSet(TagSet* tagSet)
 
     //TODO: remove actions from menus, delete actions, clear maps
 
-    QValueList<Tag> list = tagSet->toMap().values();
-    for (QValueList<Tag>::ConstIterator it = list.begin(); it != list.end(); ++it)
-        slotTagAdded(*it);
+    if (tagSet != 0L)
+    {
+        QValueList<Tag> list = tagSet->toMap().values();
+        for (QValueList<Tag>::ConstIterator it = list.begin(); it != list.end(); ++it)
+            slotTagAdded(*it);
+    }
 }
 
 void ActionManagerImpl::slotTagAdded(const Tag& tag)
@@ -230,6 +233,7 @@ ActionManagerImpl::ActionManagerImpl(Part* part, QObject* parent, const char* na
     d->articleViewer = 0;
     d->view = 0;
     d->tabWidget = 0;
+    d->tagMenu = 0;
     d->speakSelectedArticlesAction = 0;
     d->actionCollection = part->actionCollection();
     initPart();
@@ -237,9 +241,9 @@ ActionManagerImpl::ActionManagerImpl(Part* part, QObject* parent, const char* na
 
 ActionManagerImpl::~ActionManagerImpl()
 {
+    delete d->nodeSelectVisitor;
     delete d;
     d = 0;
-    delete d->nodeSelectVisitor;
 }
 
 void ActionManagerImpl::initTrayIcon(TrayIcon* trayIcon)
