@@ -59,9 +59,21 @@ namespace Akregator {
 class ArticleListView::ArticleListViewPrivate
 {
     public:
+
+    ArticleListViewPrivate(ArticleListView* parent) : m_parent(parent) { }
+
+    void ensureCurrentItemVisible()
+    {
+        if (m_parent->currentItem())
+        {
+            m_parent->center( m_parent->contentsX(), m_parent->itemPos(m_parent->currentItem()), 0, 9.0 );
+        }
+    }
+
+    ArticleListView* m_parent;
+
     /** maps article to article item */
     QMap<Article, ArticleItem*> articleMap;
-    
     TreeNode* node;
     Akregator::Filters::ArticleMatcher textFilter;
     Akregator::Filters::ArticleMatcher statusFilter;
@@ -136,7 +148,10 @@ class ArticleListView::ArticleItem : public KListViewItem
         private:
             Article m_article;
             time_t m_pubDate;
-            static QPixmap m_keepFlag;
+            static QPixmap keepFlag() {
+                   static QPixmap s_keepFlag = QPixmap(locate("data", "akregator/pics/akregator_flag.png"));
+                   return s_keepFlag;
+	    }
 };
 
 // FIXME: Remove resolveEntities for KDE 4.0, it's now done in the parser
@@ -144,7 +159,7 @@ ArticleListView::ArticleItem::ArticleItem( QListView *parent, const Article& a)
     : KListViewItem( parent, KCharsets::resolveEntities(a.title()), a.feed()->title(), KGlobal::locale()->formatDateTime(a.pubDate(), true, false) ), m_article(a), m_pubDate(a.pubDate().toTime_t())
 {
     if (a.keep())
-        setPixmap(0, m_keepFlag);
+        setPixmap(0, keepFlag());
 }
  
 ArticleListView::ArticleItem::~ArticleItem()
@@ -156,8 +171,6 @@ Article& ArticleListView::ArticleItem::article()
     return m_article;
 }
 
-QPixmap ArticleListView::ArticleItem::m_keepFlag = QPixmap(locate("data", "akregator/pics/akregator_flag.png"));
-
 // paint ze peons
 void ArticleListView::ArticleItem::paintCell ( QPainter * p, const QColorGroup & cg, int column, int width, int align )
 {
@@ -181,7 +194,7 @@ void ArticleListView::ArticleItem::paintCell ( QPainter * p, const QColorGroup &
 void ArticleListView::ArticleItem::updateItem(const Article& article)
 {
     m_article = article;
-    setPixmap(0, m_article.keep() ? m_keepFlag : QPixmap());
+    setPixmap(0, m_article.keep() ? keepFlag() : QPixmap());
     setText(0, KCharsets::resolveEntities(m_article.title()));
     setText(1, m_article.feed()->title());
     setText(2, KGlobal::locale()->formatDateTime(m_article.pubDate(), true, false));
@@ -201,8 +214,9 @@ int ArticleListView::ArticleItem::compare(QListViewItem *i, int col, bool ascend
 /* ==================================================================================== */
 
 ArticleListView::ArticleListView(QWidget *parent, const char *name)
-    : KListView(parent, name), d (new ArticleListViewPrivate)
+    : KListView(parent, name)
 {
+    d = new ArticleListViewPrivate(this);
     d->noneSelected = true;
     d->node = 0;
     d->columnMode = ArticleListViewPrivate::feedMode;
@@ -335,6 +349,9 @@ void ArticleListView::slotArticlesAdded(TreeNode* /*node*/, const QValueList<Art
 {
     setUpdatesEnabled(false);
     
+    bool statusActive = !(d->statusFilter.matchesAll());
+    bool textActive = !(d->textFilter.matchesAll());
+    
     for (QValueList<Article>::ConstIterator it = list.begin(); it != list.end(); ++it)
     {
         if (!d->articleMap.contains(*it))
@@ -342,7 +359,8 @@ void ArticleListView::slotArticlesAdded(TreeNode* /*node*/, const QValueList<Art
             if (!(*it).isNull() && !(*it).isDeleted())
             {
                 ArticleItem* ali = new ArticleItem(this, *it);
-                ali->setVisible( d->textFilter.matches( ali->article()) );
+                ali->setVisible( (!statusActive ||  d->statusFilter.matches( ali->article()))
+                        && (!textActive || d->textFilter.matches( ali->article())) );
                 d->articleMap.insert(*it, ali);
             }
         }
@@ -355,6 +373,15 @@ void ArticleListView::slotArticlesUpdated(TreeNode* /*node*/, const QValueList<A
 {
     setUpdatesEnabled(false);
 
+    // if only one item is selected and this selected item
+    // is deleted, we will select the next item in the list
+    bool singleSelected = selectedArticles().count() == 1;
+    
+    bool statusActive = !(d->statusFilter.matchesAll());
+    bool textActive = !(d->textFilter.matchesAll());
+    
+    QListViewItem* next = 0; // the item to select if a selected item is deleted
+    
     for (QValueList<Article>::ConstIterator it = list.begin(); it != list.end(); ++it)
     {
         
@@ -362,37 +389,93 @@ void ArticleListView::slotArticlesUpdated(TreeNode* /*node*/, const QValueList<A
         {
             ArticleItem* ali = d->articleMap[*it];
 
-            if ((*it).isDeleted()) // if article was set to deleted, delete item
+            if (ali)
             {
-                d->articleMap.remove(*it);
-                delete ali;
-            }
-            else
-            {
-                ali->updateItem(*it);
-                // if the updated article matches the filters after the update, make visible. If it matched them before but not after update, they should stay visible (to not confuse users)
-                if (d->textFilter.matches( ali->article()) && d->statusFilter.matches(ali->article()))
-                    ali->setVisible(true);
-            }
+                if ((*it).isDeleted()) // if article was set to deleted, delete item
+                {
+                    if (singleSelected && ali->isSelected())
+                    {
+                        if (ali->itemBelow())
+                            next = ali->itemBelow();
+                        else if (ali->itemAbove())
+                            next = ali->itemAbove();
+                    }
+                    
+                    d->articleMap.remove(*it);
+                    delete ali;
+                }
+                else
+                {
+                    ali->updateItem(*it);
+                    // if the updated article matches the filters after the update,
+                    // make visible. If it matched them before but not after update,
+                    // they should stay visible (to not confuse users)
+                    if ((!statusActive || d->statusFilter.matches(ali->article()))
+                        && (!textActive || d->textFilter.matches( ali->article())) )
+                        ali->setVisible(true);
+                }
+            } // if ali
         }
     }
 
+    // if the only selected item was deleted, select
+    // an item next to it
+    if (singleSelected && next != 0)
+    {
+        setSelected(next, true);
+        setCurrentItem(next);
+    }
+    else
+    {
+        d->noneSelected = true;
+    }
+    
+
     setUpdatesEnabled(true);
     triggerUpdate();
 }
 
 void ArticleListView::slotArticlesRemoved(TreeNode* /*node*/, const QValueList<Article>& list)
 {
+    // if only one item is selected and this selected item
+    // is deleted, we will select the next item in the list
+    bool singleSelected = selectedArticles().count() == 1;
+
+    QListViewItem* next = 0; // the item to select if a selected item is deleted
+    
     setUpdatesEnabled(false);
+    
     for (QValueList<Article>::ConstIterator it = list.begin(); it != list.end(); ++it)
     {
         if (d->articleMap.contains(*it))
         {
             ArticleItem* ali = d->articleMap[*it];
             d->articleMap.remove(*it);
+            
+            if (singleSelected && ali->isSelected())
+            {
+                if (ali->itemBelow())
+                    next = ali->itemBelow();
+                else if (ali->itemAbove())
+                    next = ali->itemAbove();
+            }
+            
             delete ali;
         }
     }
+    
+    // if the only selected item was deleted, select
+    // an item next to it
+    if (singleSelected && next != 0)
+    {
+        setSelected(next, true);
+        setCurrentItem(next);
+    }
+    else
+    {
+        d->noneSelected = true;
+    }
+     
     setUpdatesEnabled(true);
     triggerUpdate();
 }
@@ -415,12 +498,44 @@ void ArticleListView::disconnectFromNode(TreeNode* node)
 
 void ArticleListView::applyFilters()
 {
+    bool statusActive = !(d->statusFilter.matchesAll());
+    bool textActive = !(d->textFilter.matchesAll());
+    
     ArticleItem* ali = 0;
-    for (QListViewItemIterator it(this); it.current(); ++it)
+    
+    if (!statusActive && !textActive)
     {
-        ali = static_cast<ArticleItem*> (it.current());
-        ali->setVisible( d->statusFilter.matches( ali->article() ) && d->textFilter.matches( ali->article() ) );
+        for (QListViewItemIterator it(this); it.current(); ++it)
+        {
+            (static_cast<ArticleItem*> (it.current()))->setVisible(true);
+        }
     }
+    else if (statusActive && !textActive)
+    {
+        for (QListViewItemIterator it(this); it.current(); ++it)
+        {
+            ali = static_cast<ArticleItem*> (it.current());
+            ali->setVisible( d->statusFilter.matches( ali->article()) );
+        }
+    }
+    else if (!statusActive && textActive)
+    {
+        for (QListViewItemIterator it(this); it.current(); ++it)
+        {
+            ali = static_cast<ArticleItem*> (it.current());
+            ali->setVisible( d->textFilter.matches( ali->article()) );
+        }
+    }
+    else // both true
+    {
+        for (QListViewItemIterator it(this); it.current(); ++it)
+        {
+            ali = static_cast<ArticleItem*> (it.current());
+            ali->setVisible( d->statusFilter.matches( ali->article()) 
+                    && d->textFilter.matches( ali->article()) );
+        }
+    }
+
 }
 
 int ArticleListView::visibleArticles()
@@ -514,17 +629,17 @@ void ArticleListView::slotPreviousArticle()
 {
     ArticleItem* ali = 0;
     if (!currentItem() || selectedItems().isEmpty())
-        ali = dynamic_cast<ArticleItem*>(firstChild());
+        ali = dynamic_cast<ArticleItem*>(lastChild());
     else
         ali = dynamic_cast<ArticleItem*>(currentItem()->itemAbove());
     
     if (ali)
     {
         Article a = ali->article();
+        setCurrentItem(d->articleMap[a]);
         clearSelection();
         setSelected(d->articleMap[a], true);
-        setCurrentItem(d->articleMap[a]);
-        ensureItemVisible(d->articleMap[a]);
+        d->ensureCurrentItemVisible();
     }
 }
 
@@ -539,30 +654,35 @@ void ArticleListView::slotNextArticle()
     if (ali)
     {
         Article a = ali->article();
+        setCurrentItem(d->articleMap[a]);
         clearSelection();
         setSelected(d->articleMap[a], true);
-        setCurrentItem(d->articleMap[a]);
-        ensureItemVisible(d->articleMap[a]);
+        d->ensureCurrentItemVisible();
     }
 }
 
 void ArticleListView::slotNextUnreadArticle()
 {
-    ArticleItem* start = 0;
+    ArticleItem* start = 0L;
     if (!currentItem() || selectedItems().isEmpty())
         start = dynamic_cast<ArticleItem*>(firstChild());
     else
         start = dynamic_cast<ArticleItem*>(currentItem()->itemBelow() ? currentItem()->itemBelow() : firstChild());
 
     ArticleItem* i = start;
-    ArticleItem* unread = 0;
+    ArticleItem* unread = 0L;
     
     do
     {
-        if (i && i->article().status() != Article::Read)
-            unread = i;
-        else 
-            i = dynamic_cast<ArticleItem*>(i && i->itemBelow() ? i->itemBelow() : firstChild());
+        if (i == 0L)
+            i = static_cast<ArticleItem*>(firstChild());
+        else
+        {
+            if (i->article().status() != Article::Read)
+                unread = i;
+            else 
+                i = static_cast<ArticleItem*>(i && i->itemBelow() ? i->itemBelow() : firstChild());
+        }
     }
     while (!unread && i != start);
 
@@ -572,29 +692,34 @@ void ArticleListView::slotNextUnreadArticle()
         setCurrentItem(d->articleMap[a]);
         clearSelection();
         setSelected(d->articleMap[a], true);
-        ensureItemVisible(d->articleMap[a]);
+        d->ensureCurrentItemVisible();
     }
 }
 
 void ArticleListView::slotPreviousUnreadArticle()
 {
-    ArticleItem* start = 0;
+    ArticleItem* start = 0L;
     if (!currentItem() || selectedItems().isEmpty())
-        start = dynamic_cast<ArticleItem*>(firstChild());
+        start = dynamic_cast<ArticleItem*>(lastChild());
     else
         start = dynamic_cast<ArticleItem*>(currentItem()->itemAbove() ? currentItem()->itemAbove() : firstChild());
 
     ArticleItem* i = start;
-    ArticleItem* unread = 0;
+    ArticleItem* unread = 0L;
     
     do
     {
-        if (i && i->article().status() != Article::Read)
-            unread = i;
-        else 
-            i = dynamic_cast<ArticleItem*>(i->itemAbove() ? i->itemAbove() : lastChild());
+        if (i == 0L)
+            i = static_cast<ArticleItem*>(lastChild());
+        else
+        {
+            if (i->article().status() != Article::Read)
+                unread = i;
+            else 
+                i = static_cast<ArticleItem*>(i->itemAbove() ? i->itemAbove() : lastChild());
+        }
     }
-    while ( !(unread != 0 || i == start) );
+    while ( !(unread != 0L || i == start) );
 
     if (unread)
     {
@@ -602,7 +727,7 @@ void ArticleListView::slotPreviousUnreadArticle()
         setCurrentItem(d->articleMap[a]);
         clearSelection();
         setSelected(d->articleMap[a], true);
-        ensureItemVisible(d->articleMap[a]);
+        d->ensureCurrentItemVisible();
     }
 }
 
@@ -630,8 +755,11 @@ void ArticleListView::slotCurrentChanged(QListViewItem* item)
     ArticleItem* ai = dynamic_cast<ArticleItem*> (item);
     if (ai)
         emit signalArticleChosen( ai->article() );
-    else 
+    else
+    {
+        d->noneSelected = true;
         emit signalArticleChosen( Article() );
+    }
 } 
 
 
