@@ -306,7 +306,7 @@ void QGpgMECryptoConfigComponent::sync( bool runtime )
   }
   else if( rc != 0 ) // Happens due to bugs in gpgconf (e.g. issues 104/115)
   {
-    QString wmsg = i18n( "Error from gpgconf while saving configuration: %1" ).arg( strerror( rc ) );
+    QString wmsg = i18n( "Error from gpgconf while saving configuration: %1" ).arg( QString::fromLocal8Bit( strerror( rc ) ) );
     kdWarning(5150) << k_funcinfo << ":" << strerror( rc ) << endl;
     KMessageBox::error(0, wmsg);
   }
