@@ -1,7 +1,7 @@
 /*
  *  editdlg.cpp  -  dialogue to create or modify an alarm or alarm template
  *  Program:  kalarm
- *  Copyright © 2001-2006 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright © 2001-2008 by David Jarvie <djarvie@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -76,7 +76,6 @@
 #include "radiobutton.h"
 #include "recurrenceedit.h"
 #include "reminder.h"
-#include "repetition.h"
 #include "shellprocess.h"
 #include "soundpicker.h"
 #include "specialactions.h"
@@ -128,7 +127,12 @@ class PickLogFileRadio : public PickFileRadio
 
 inline QString recurText(const KAEvent& event)
 {
-	return QString::fromLatin1("%1 / %2").arg(event.recurrenceText()).arg(event.repetitionText());
+	QString r;
+	if (event.repeatCount())
+		r = QString::fromLatin1("%1 / %2").arg(event.recurrenceText()).arg(event.repetitionText());
+	else
+		r = event.recurrenceText();
+	return i18n("&Recurrence - [%1]").arg(r);
 }
 
 // Collect these widget labels together to ensure consistent wording and
@@ -163,7 +167,7 @@ QString EditAlarmDlg::i18n_j_EmailSubject()     { return i18n("Email subject", "
  */
 EditAlarmDlg::EditAlarmDlg(bool Template, const QString& caption, QWidget* parent, const char* name,
                            const KAEvent* event, bool readOnly)
-	: KDialogBase(parent, name, true, caption,
+	: KDialogBase(parent, (name ? name : Template ? "TemplEditDlg" : "EditDlg"), true, caption,
 	              (readOnly ? Cancel|Try : Template ? Ok|Cancel|Try : Ok|Cancel|Try|Default),
 	              (readOnly ? Cancel : Ok)),
 	  mMainPageShown(false),
@@ -213,20 +217,22 @@ EditAlarmDlg::EditAlarmDlg(bool Template, const QString& caption, QWidget* paren
 	// Recurrence tab
 	QVBox* recurTab = new QVBox(mTabs);
 	mainPageBox->setSpacing(spacingHint());
-	mTabs->addTab(recurTab, i18n("&Recurrence"));
+	mTabs->addTab(recurTab, QString::null);
 	mRecurPageIndex = 1;
 	mRecurrenceEdit = new RecurrenceEdit(readOnly, recurTab, "recurPage");
 	connect(mRecurrenceEdit, SIGNAL(shown()), SLOT(slotShowRecurrenceEdit()));
 	connect(mRecurrenceEdit, SIGNAL(typeChanged(int)), SLOT(slotRecurTypeChange(int)));
 	connect(mRecurrenceEdit, SIGNAL(frequencyChanged()), SLOT(slotRecurFrequencyChange()));
+	connect(mRecurrenceEdit, SIGNAL(repeatNeedsInitialisation()), SLOT(slotSetSubRepetition()));
 
 	// Alarm action
 
 	mActionGroup = new ButtonGroup(i18n("Action"), mainPage, "actionGroup");
 	connect(mActionGroup, SIGNAL(buttonSet(int)), SLOT(slotAlarmTypeChanged(int)));
 	topLayout->addWidget(mActionGroup, 1);
-	QGridLayout* grid = new QGridLayout(mActionGroup, 3, 5, marginHint(), spacingHint());
-	grid->addRowSpacing(0, fontMetrics().lineSpacing()/2);
+	QBoxLayout* layout = new QVBoxLayout(mActionGroup, marginHint(), spacingHint());
+	layout->addSpacing(fontMetrics().lineSpacing()/2);
+	QGridLayout* grid = new QGridLayout(layout, 1, 5);
 
 	// Message radio button
 	mMessageRadio = new RadioButton(i18n("Te&xt"), mActionGroup, "messageButton");
@@ -260,14 +266,11 @@ EditAlarmDlg::EditAlarmDlg(bool Template, const QString& caption, QWidget* paren
 	grid->addWidget(mEmailRadio, 1, 6);
 
 	initDisplayAlarms(mActionGroup);
+	layout->addWidget(mDisplayAlarmsFrame);
 	initCommand(mActionGroup);
+	layout->addWidget(mCommandFrame);
 	initEmail(mActionGroup);
-	mAlarmTypeStack = new QWidgetStack(mActionGroup);
-	grid->addMultiCellWidget(mAlarmTypeStack, 2, 2, 0, 6);
-	grid->setRowStretch(2, 1);
-	mAlarmTypeStack->addWidget(mDisplayAlarmsFrame, 0);
-	mAlarmTypeStack->addWidget(mCommandFrame, 1);
-	mAlarmTypeStack->addWidget(mEmailFrame, 2);
+	layout->addWidget(mEmailFrame);
 
 	// Deferred date/time: visible only for a deferred recurring event.
 	mDeferGroup = new QGroupBox(1, Qt::Vertical, i18n("Deferred Alarm"), mainPage, "deferGroup");
@@ -282,7 +285,7 @@ EditAlarmDlg::EditAlarmDlg(bool Template, const QString& caption, QWidget* paren
 	QWhatsThis::add(mDeferChangeButton, i18n("Change the alarm's deferred time, or cancel the deferral"));
 	mDeferGroup->addSpace(0);
 
-	QBoxLayout* layout = new QHBoxLayout(topLayout);
+	layout = new QHBoxLayout(topLayout);
 
 	// Date and time entry
 	if (mTemplate)
@@ -356,27 +359,6 @@ EditAlarmDlg::EditAlarmDlg(bool Template, const QString& caption, QWidget* paren
 		topLayout->addWidget(mTimeWidget);
 	}
 
-	// Recurrence type display
-	layout = new QHBoxLayout(topLayout, 2*spacingHint());
-	QHBox* box = new QHBox(mainPage);   // this is to control the QWhatsThis text display area
-	box->setSpacing(KDialog::spacingHint());
-	label = new QLabel(i18n("Recurrence:"), box);
-	label->setFixedSize(label->sizeHint());
-	mRecurrenceText = new QLabel(box);
-	QWhatsThis::add(box,
-	      i18n("How often the alarm recurs.\nThe times shown are those configured in the Recurrence tab and in the Simple Repetition dialog."));
-	box->setFixedHeight(box->sizeHint().height());
-	layout->addWidget(box);
-	layout->addStretch();
-
-	// Simple repetition button
-	mSimpleRepetition = new RepetitionButton(i18n("Simple Repetition"), true, mainPage);
-	mSimpleRepetition->setFixedSize(mSimpleRepetition->sizeHint());
-	connect(mSimpleRepetition, SIGNAL(needsInitialisation()), SLOT(slotSetSimpleRepetition()));
-	connect(mSimpleRepetition, SIGNAL(changed()), SLOT(slotRecurFrequencyChange()));
-	QWhatsThis::add(mSimpleRepetition, i18n("Set up a simple, or additional, alarm repetition"));
-	layout->addWidget(mSimpleRepetition);
-
 	// Reminder
 	static const QString reminderText = i18n("Enter how long in advance of the main alarm to display a reminder alarm.");
 	mReminder = new Reminder(i18n("Rem&inder:"),
@@ -390,13 +372,21 @@ EditAlarmDlg::EditAlarmDlg(bool Template, const QString& caption, QWidget* paren
 	mLateCancel = new LateCancelSelector(true, mainPage);
 	topLayout->addWidget(mLateCancel, 0, Qt::AlignAuto);
 
+	// Acknowledgement confirmation required - default = no confirmation
+	layout = new QHBoxLayout(topLayout, 0);
+	mConfirmAck = createConfirmAckCheckbox(mainPage);
+	mConfirmAck->setFixedSize(mConfirmAck->sizeHint());
+	layout->addWidget(mConfirmAck);
+	layout->addSpacing(2*spacingHint());
+	layout->addStretch();
+
 	if (theApp()->korganizerEnabled())
 	{
 		// Show in KOrganizer checkbox
 		mShowInKorganizer = new CheckBox(i18n_ShowInKOrganizer(), mainPage);
 		mShowInKorganizer->setFixedSize(mShowInKorganizer->sizeHint());
 		QWhatsThis::add(mShowInKorganizer, i18n("Check to copy the alarm into KOrganizer's calendar"));
-		topLayout->addWidget(mShowInKorganizer);
+		layout->addWidget(mShowInKorganizer);
 	}
 
 	setButtonWhatsThis(Ok, i18n("Schedule the alarm at the specified time."));
@@ -432,7 +422,7 @@ void EditAlarmDlg::initDisplayAlarms(QWidget* parent)
 
 	// Text message edit box
 	mTextMessageEdit = new TextEdit(mDisplayAlarmsFrame);
-	mTextMessageEdit->setWordWrap(QTextEdit::NoWrap);
+	mTextMessageEdit->setWordWrap(KTextEdit::NoWrap);
 	QWhatsThis::add(mTextMessageEdit, i18n("Enter the text of the alarm message. It may be multi-line."));
 	frameLayout->addWidget(mTextMessageEdit);
 
@@ -451,33 +441,17 @@ void EditAlarmDlg::initDisplayAlarms(QWidget* parent)
 	QWhatsThis::add(mFileBrowseButton, i18n("Select a text or image file to display."));
 	mFileRadio->init(mFileBrowseButton, mFileMessageEdit);
 
-	// Colour choice drop-down list
-	QBoxLayout* layout = new QHBoxLayout(frameLayout);
-	QHBox* box;
-	mBgColourChoose = createBgColourChooser(&box, mDisplayAlarmsFrame);
-	mBgColourChoose->setFixedSize(mBgColourChoose->sizeHint());
-	connect(mBgColourChoose, SIGNAL(highlighted(const QColor&)), SLOT(slotBgColourSelected(const QColor&)));
-	layout->addWidget(box);
-	layout->addSpacing(2*KDialog::spacingHint());
-	layout->addStretch();
-
-	// Font and colour choice drop-down list
+	// Font and colour choice button and sample text
 	mFontColourButton = new FontColourButton(mDisplayAlarmsFrame);
-	mFontColourButton->setFixedSize(mFontColourButton->sizeHint());
-	connect(mFontColourButton, SIGNAL(selected()), SLOT(slotFontColourSelected()));
-	layout->addWidget(mFontColourButton);
+	mFontColourButton->setMaximumHeight(mFontColourButton->sizeHint().height());
+	frameLayout->addWidget(mFontColourButton);
 
 	// Sound checkbox and file selector
+	QHBoxLayout* layout = new QHBoxLayout(frameLayout);
 	mSoundPicker = new SoundPicker(mDisplayAlarmsFrame);
 	mSoundPicker->setFixedSize(mSoundPicker->sizeHint());
-	frameLayout->addWidget(mSoundPicker, 0, Qt::AlignAuto);
-
-	// Acknowledgement confirmation required - default = no confirmation
-	layout = new QHBoxLayout(frameLayout);
-	mConfirmAck = createConfirmAckCheckbox(mDisplayAlarmsFrame);
-	mConfirmAck->setFixedSize(mConfirmAck->sizeHint());
-	layout->addWidget(mConfirmAck);
-	layout->addSpacing(2*KDialog::spacingHint());
+	layout->addWidget(mSoundPicker);
+	layout->addSpacing(2*spacingHint());
 	layout->addStretch();
 
 	if (ShellProcess::authorised())    // don't display if shell commands not allowed (e.g. kiosk mode)
@@ -702,7 +676,6 @@ void EditAlarmDlg::initialise(const KAEvent* event)
 			mFontColourButton->setFont(event->font());
 		mFontColourButton->setBgColour(event->bgColour());
 		mFontColourButton->setFgColour(event->fgColour());
-		mBgColourChoose->setColour(event->bgColour());     // set colour before setting alarm type buttons
 		if (mTemplate)
 		{
 			// Editing a template
@@ -786,14 +759,14 @@ void EditAlarmDlg::initialise(const KAEvent* event)
 		mReminder->enableOnceOnly(event->recurs());
 		if (mSpecialActionsButton)
 			mSpecialActionsButton->setActions(event->preAction(), event->postAction());
-		mSimpleRepetition->set(event->repeatInterval(), event->repeatCount());
-		mRecurrenceText->setText(recurText(*event));
 		mRecurrenceEdit->set(*event);   // must be called after mTimeWidget is set up, to ensure correct date-only enabling
-		SoundPicker::Type soundType = event->speak()                                ? SoundPicker::SPEAK
-		                            : event->beep() || event->audioFile().isEmpty() ? SoundPicker::BEEP
-		                            :                                                 SoundPicker::PLAY_FILE;
-		mSoundPicker->set((soundType != SoundPicker::BEEP || event->beep()), soundType, event->audioFile(),
-		                  event->soundVolume(), event->fadeVolume(), event->fadeSeconds(), event->repeatSound());
+		mTabs->setTabLabel(mTabs->page(mRecurPageIndex), recurText(*event));
+		SoundPicker::Type soundType = event->speak()                ? SoundPicker::SPEAK
+		                            : event->beep()                 ? SoundPicker::BEEP
+		                            : !event->audioFile().isEmpty() ? SoundPicker::PLAY_FILE
+		                            :                                 SoundPicker::NONE;
+		mSoundPicker->set(soundType, event->audioFile(), event->soundVolume(),
+		                  event->fadeVolume(), event->fadeSeconds(), event->repeatSound());
 		CmdLogType logType = event->commandXterm()       ? EXEC_IN_TERMINAL
 		                   : !event->logFile().isEmpty() ? LOG_TO_FILE
 		                   :                               DISCARD_OUTPUT;
@@ -804,7 +777,7 @@ void EditAlarmDlg::initialise(const KAEvent* event)
 		mEmailSubjectEdit->setText(event->emailSubject());
 		mEmailBcc->setChecked(event->emailBcc());
 		if (mEmailFromList)
-			mEmailFromList->setCurrentIdentity(event->emailFromKMail());
+			mEmailFromList->setCurrentIdentity(event->emailFromId());
 	}
 	else
 	{
@@ -819,7 +792,6 @@ void EditAlarmDlg::initialise(const KAEvent* event)
 		mFontColourButton->setDefaultFont();
 		mFontColourButton->setBgColour(Preferences::defaultBgColour());
 		mFontColourButton->setFgColour(Preferences::defaultFgColour());
-		mBgColourChoose->setColour(Preferences::defaultBgColour());     // set colour before setting alarm type buttons
 		QDateTime defaultTime = QDateTime::currentDateTime().addSecs(60);
 		if (mTemplate)
 		{
@@ -843,7 +815,7 @@ void EditAlarmDlg::initialise(const KAEvent* event)
 		slotRecurFrequencyChange();      // update the Recurrence text
 		mReminder->setMinutes(0, false);
 		mReminder->enableOnceOnly(mRecurrenceEdit->isTimedRepeatType());   // must be called after mRecurrenceEdit is set up
-		mSoundPicker->set(Preferences::defaultSound(), Preferences::defaultSoundType(), Preferences::defaultSoundFile(),
+		mSoundPicker->set(Preferences::defaultSoundType(), Preferences::defaultSoundFile(),
 		                  Preferences::defaultSoundVolume(), -1, 0, Preferences::defaultSoundRepeat());
 		mCmdTypeScript->setChecked(Preferences::defaultCmdScript());
 		mCmdLogFileEdit->setText(Preferences::defaultCmdLogFile());    // set file name before setting radio button
@@ -881,14 +853,12 @@ void EditAlarmDlg::setReadOnly()
 		mDeferChangeButton->hide();
 	else
 		mDeferChangeButton->show();
-	mSimpleRepetition->setReadOnly(mReadOnly);
 	if (mShowInKorganizer)
 		mShowInKorganizer->setReadOnly(mReadOnly);
 
 	// Message alarm controls
 	mTextMessageEdit->setReadOnly(mReadOnly);
 	mFileMessageEdit->setReadOnly(mReadOnly);
-	mBgColourChoose->setReadOnly(mReadOnly);
 	mFontColourButton->setReadOnly(mReadOnly);
 	mSoundPicker->setReadOnly(mReadOnly);
 	mConfirmAck->setReadOnly(mReadOnly);
@@ -986,24 +956,6 @@ void EditAlarmDlg::setAction(KAEvent::Action action, const AlarmText& alarmText)
 	mActionGroup->setButton(mActionGroup->id(radio));
 }
 
-/******************************************************************************
- * Create a widget to choose the alarm message background colour.
- */
-ColourCombo* EditAlarmDlg::createBgColourChooser(QHBox** box, QWidget* parent, const char* name)
-{
-	*box = new QHBox(parent);   // this is to control the QWhatsThis text display area
-	QLabel* label = new QLabel(i18n("&Background color:"), *box);
-	label->setFixedSize(label->sizeHint());
-	ColourCombo* widget = new ColourCombo(*box, name);
-	QSize size = widget->sizeHint();
-	widget->setMinimumHeight(size.height() + 4);
-	QToolTip::add(widget, i18n("Message color"));
-	label->setBuddy(widget);
-	(*box)->setFixedHeight((*box)->sizeHint().height());
-	QWhatsThis::add(*box, i18n("Choose the background color for the alarm message."));
-	return widget;
-}
-
 /******************************************************************************
  * Create an "acknowledgement confirmation required" checkbox.
  */
@@ -1020,6 +972,7 @@ CheckBox* EditAlarmDlg::createConfirmAckCheckbox(QWidget* parent, const char* na
  */
 void EditAlarmDlg::saveState(const KAEvent* event)
 {
+	delete mSavedEvent;
 	mSavedEvent = 0;
 	if (event)
 		mSavedEvent = new KAEvent(*event);
@@ -1031,15 +984,14 @@ void EditAlarmDlg::saveState(const KAEvent* event)
 		mSavedTemplateAfterTime = mTemplateTimeAfter->value();
 	}
 	mSavedTypeRadio        = mActionGroup->selected();
-	mSavedSound            = mSoundPicker->sound();
-	mSavedSoundType        = mSoundPicker->type();
+	mSavedSoundType        = mSoundPicker->sound();
 	mSavedSoundFile        = mSoundPicker->file();
 	mSavedSoundVolume      = mSoundPicker->volume(mSavedSoundFadeVolume, mSavedSoundFadeSeconds);
 	mSavedRepeatSound      = mSoundPicker->repeat();
 	mSavedConfirmAck       = mConfirmAck->isChecked();
 	mSavedFont             = mFontColourButton->font();
 	mSavedFgColour         = mFontColourButton->fgColour();
-	mSavedBgColour         = mBgColourChoose->color();
+	mSavedBgColour         = mFontColourButton->bgColour();
 	mSavedReminder         = mReminder->minutes();
 	mSavedOnceOnly         = mReminder->isOnceOnly();
 	if (mSpecialActionsButton)
@@ -1066,8 +1018,6 @@ void EditAlarmDlg::saveState(const KAEvent* event)
 	if (mShowInKorganizer)
 		mSavedShowInKorganizer = mShowInKorganizer->isChecked();
 	mSavedRecurrenceType   = mRecurrenceEdit->repeatType();
-	mSavedRepeatInterval   = mSimpleRepetition->interval();
-	mSavedRepeatCount      = mSimpleRepetition->count();
 }
 
 /******************************************************************************
@@ -1100,17 +1050,15 @@ bool EditAlarmDlg::stateChanged() const
 	||  mSavedLateCancel       != mLateCancel->minutes()
 	||  mShowInKorganizer && mSavedShowInKorganizer != mShowInKorganizer->isChecked()
 	||  textFileCommandMessage != mSavedTextFileCommandMessage
-	||  mSavedRecurrenceType   != mRecurrenceEdit->repeatType()
-	||  mSavedRepeatInterval   != mSimpleRepetition->interval()
-	||  mSavedRepeatCount      != mSimpleRepetition->count())
+	||  mSavedRecurrenceType   != mRecurrenceEdit->repeatType())
 		return true;
 	if (mMessageRadio->isOn()  ||  mFileRadio->isOn())
 	{
-		if (mSavedSound      != mSoundPicker->sound()
+		if (mSavedSoundType  != mSoundPicker->sound()
 		||  mSavedConfirmAck != mConfirmAck->isChecked()
 		||  mSavedFont       != mFontColourButton->font()
 		||  mSavedFgColour   != mFontColourButton->fgColour()
-		||  mSavedBgColour   != mBgColourChoose->color()
+		||  mSavedBgColour   != mFontColourButton->bgColour()
 		||  mSavedReminder   != mReminder->minutes()
 		||  mSavedOnceOnly   != mReminder->isOnceOnly()
 		||  mSavedAutoClose  != mLateCancel->isAutoClose())
@@ -1121,24 +1069,19 @@ bool EditAlarmDlg::stateChanged() const
 			||  mSavedPostAction != mSpecialActionsButton->postAction())
 				return true;
 		}
-		if (mSavedSound)
+		if (mSavedSoundType == SoundPicker::PLAY_FILE)
 		{
-			if (mSavedSoundType != mSoundPicker->type())
+			if (mSavedSoundFile != mSoundPicker->file())
 				return true;
-			if (mSavedSoundType == SoundPicker::PLAY_FILE)
+			if (!mSavedSoundFile.isEmpty())
 			{
-				if (mSavedSoundFile != mSoundPicker->file())
+				float fadeVolume;
+				int   fadeSecs;
+				if (mSavedRepeatSound != mSoundPicker->repeat()
+				||  mSavedSoundVolume != mSoundPicker->volume(fadeVolume, fadeSecs)
+				||  mSavedSoundFadeVolume != fadeVolume
+				||  mSavedSoundFadeSeconds != fadeSecs)
 					return true;
-				if (!mSavedSoundFile.isEmpty())
-				{
-					float fadeVolume;
-					int   fadeSecs;
-					if (mSavedRepeatSound != mSoundPicker->repeat()
-					||  mSavedSoundVolume != mSoundPicker->volume(fadeVolume, fadeSecs)
-					||  mSavedSoundFadeVolume != fadeVolume
-					||  mSavedSoundFadeSeconds != fadeSecs)
-						return true;
-				}
 			}
 		}
 	}
@@ -1214,10 +1157,10 @@ void EditAlarmDlg::setEvent(KAEvent& event, const QString& text, bool trial)
 		if (!mTemplate)
 			dt = mAlarmDateTime.dateTime();
 		else if (mTemplateUseTime->isOn())
-			dt.setTime(mTemplateTime->time());
+			dt = QDateTime(QDate(2000,1,1), mTemplateTime->time());
 	}
 	KAEvent::Action type = getAlarmType();
-	event.set(dt, text, mBgColourChoose->color(), mFontColourButton->fgColour(), mFontColourButton->font(),
+	event.set(dt, text, mFontColourButton->bgColour(), mFontColourButton->fgColour(), mFontColourButton->font(),
 	          type, (trial ? 0 : mLateCancel->minutes()), getAlarmFlags());
 	switch (type)
 	{
@@ -1239,9 +1182,7 @@ void EditAlarmDlg::setEvent(KAEvent& event, const QString& text, bool trial)
 		}
 		case KAEvent::EMAIL:
 		{
-			QString from;
-			if (mEmailFromList)
-				from = mEmailFromList->currentIdentityName();
+			uint from = mEmailFromList ? mEmailFromList->currentIdentity() : 0;
 			event.setEmail(from, mEmailAddresses, mEmailSubjectEdit->text(), mEmailAttachments);
 			break;
 		}
@@ -1257,6 +1198,15 @@ void EditAlarmDlg::setEvent(KAEvent& event, const QString& text, bool trial)
 		if (mRecurrenceEdit->repeatType() != RecurrenceEdit::NO_RECUR)
 		{
 			mRecurrenceEdit->updateEvent(event, !mTemplate);
+			QDateTime now = QDateTime::currentDateTime();
+			bool dateOnly = mAlarmDateTime.isDateOnly();
+			if (dateOnly  &&  mAlarmDateTime.date() < now.date()
+			||  !dateOnly  &&  mAlarmDateTime.rawDateTime() < now)
+			{
+				// A timed recurrence has an entered start date which has
+				// already expired, so we must adjust the next repetition.
+				event.setNextOccurrence(now);
+			}
 			mAlarmDateTime = event.startDateTime();
 			if (mDeferDateTime.isValid()  &&  mDeferDateTime < mAlarmDateTime)
 			{
@@ -1278,8 +1228,6 @@ void EditAlarmDlg::setEvent(KAEvent& event, const QString& text, bool trial)
 					event.defer(mDeferDateTime, deferReminder, false);
 			}
 		}
-		if (mSimpleRepetition->count())
-			event.setRepetition(mSimpleRepetition->interval(), mSimpleRepetition->count());
 		if (mTemplate)
 		{
 			int afterTime = mTemplateDefaultTime->isOn() ? 0
@@ -1297,8 +1245,8 @@ int EditAlarmDlg::getAlarmFlags() const
 	bool displayAlarm = mMessageRadio->isOn() || mFileRadio->isOn();
 	bool cmdAlarm     = mCommandRadio->isOn();
 	bool emailAlarm   = mEmailRadio->isOn();
-	return (displayAlarm && mSoundPicker->beep()                                 ? KAEvent::BEEP : 0)
-	     | (displayAlarm && mSoundPicker->speak()                                ? KAEvent::SPEAK : 0)
+	return (displayAlarm && mSoundPicker->sound() == SoundPicker::BEEP           ? KAEvent::BEEP : 0)
+	     | (displayAlarm && mSoundPicker->sound() == SoundPicker::SPEAK          ? KAEvent::SPEAK : 0)
 	     | (displayAlarm && mSoundPicker->repeat()                               ? KAEvent::REPEAT_SOUND : 0)
 	     | (displayAlarm && mConfirmAck->isChecked()                             ? KAEvent::CONFIRM_ACK : 0)
 	     | (displayAlarm && mLateCancel->isAutoClose()                           ? KAEvent::AUTO_CLOSE : 0)
@@ -1379,7 +1327,6 @@ void EditAlarmDlg::slotOk()
 	&&  mTabs->currentPageIndex() == mRecurPageIndex  &&  recurType == RecurrenceEdit::AT_LOGIN)
 		mTimeWidget->setDateTime(mRecurrenceEdit->endDateTime());
 	bool timedRecurrence = mRecurrenceEdit->isTimedRepeatType();    // does it recur other than at login?
-	bool repeated = mSimpleRepetition->count();
 	if (mTemplate)
 	{
 		// Check that the template name is not blank and is unique
@@ -1403,7 +1350,7 @@ void EditAlarmDlg::slotOk()
 	else
 	{
 		QWidget* errWidget;
-		mAlarmDateTime = mTimeWidget->getDateTime(0, !(timedRecurrence || repeated), false, &errWidget);
+		mAlarmDateTime = mTimeWidget->getDateTime(0, !timedRecurrence, false, &errWidget);
 		if (errWidget)
 		{
 			// It's more than just an existing deferral being changed, so the time matters
@@ -1449,58 +1396,46 @@ void EditAlarmDlg::slotOk()
 			return;
 		}
 	}
-	int longestRecurInterval = -1;
 	if (recurType != RecurrenceEdit::NO_RECUR)
 	{
+		KAEvent recurEvent;
+		int longestRecurInterval = -1;
 		int reminder = mReminder->minutes();
-		if (reminder)
+		if (reminder  &&  !mReminder->isOnceOnly())
 		{
-			KAEvent event;
-			mRecurrenceEdit->updateEvent(event, false);
-			if (!mReminder->isOnceOnly())
+			getEvent(recurEvent);     // this may adjust mAlarmDateTime
+			mRecurrenceEdit->updateEvent(recurEvent, false);
+			longestRecurInterval = recurEvent.longestRecurrenceInterval();
+			if (longestRecurInterval  &&  reminder >= longestRecurInterval)
 			{
-				longestRecurInterval = event.longestRecurrenceInterval();
-				if (longestRecurInterval  &&  reminder >= longestRecurInterval)
-				{
-					mTabs->setCurrentPage(mMainPageIndex);
-					mReminder->setFocusOnCount();
-					KMessageBox::sorry(this, i18n("Reminder period must be less than the recurrence interval, unless '%1' is checked."
-					                             ).arg(Reminder::i18n_first_recurrence_only()));
-					return;
-				}
+				mTabs->setCurrentPage(mMainPageIndex);
+				mReminder->setFocusOnCount();
+				KMessageBox::sorry(this, i18n("Reminder period must be less than the recurrence interval, unless '%1' is checked."
+				                             ).arg(Reminder::i18n_first_recurrence_only()));
+				return;
 			}
 		}
-	}
-	if (mSimpleRepetition->count())
-	{
-		switch (recurType)
+		if (mRecurrenceEdit->subRepeatCount())
 		{
-			case RecurrenceEdit::AT_LOGIN:    // alarm repeat not allowed
-				mSimpleRepetition->set(0, 0);
-				break;
-			default:          // repeat duration must be less than recurrence interval
-				if (longestRecurInterval < 0)
-				{
-					KAEvent event;
-					mRecurrenceEdit->updateEvent(event, false);
-					longestRecurInterval = event.longestRecurrenceInterval();
-				}
-				if (mSimpleRepetition->count() * mSimpleRepetition->interval() >= longestRecurInterval - mReminder->minutes())
-				{
-					KMessageBox::sorry(this, i18n("Simple alarm repetition duration must be less than the recurrence interval minus any reminder period"));
-					mSimpleRepetition->activate();   // display the alarm repetition dialog again
-					return;
-				}
-				// fall through to NO_RECUR
-			case RecurrenceEdit::NO_RECUR:    // no restriction on repeat duration
-				if (mSimpleRepetition->interval() % 1440
-				&&  (mTemplate && mTemplateAnyTime->isOn()  ||  !mTemplate && mAlarmDateTime.isDateOnly()))
-				{
-					KMessageBox::sorry(this, i18n("Simple alarm repetition period must be in units of days or weeks for a date-only alarm"));
-					mSimpleRepetition->activate();   // display the alarm repetition dialog again
-					return;
-				}
-				break;
+			if (longestRecurInterval < 0)
+			{
+				getEvent(recurEvent);     // this may adjust mAlarmDateTime
+				mRecurrenceEdit->updateEvent(recurEvent, false);
+				longestRecurInterval = recurEvent.longestRecurrenceInterval();
+			}
+			if (recurEvent.repeatInterval() * recurEvent.repeatCount() >= longestRecurInterval - reminder)
+			{
+				KMessageBox::sorry(this, i18n("The duration of a repetition within the recurrence must be less than the recurrence interval minus any reminder period"));
+				mRecurrenceEdit->activateSubRepetition();   // display the alarm repetition dialog again
+				return;
+			}
+			if (recurEvent.repeatInterval() % 1440
+			&&  (mTemplate && mTemplateAnyTime->isOn()  ||  !mTemplate && mAlarmDateTime.isDateOnly()))
+			{
+				KMessageBox::sorry(this, i18n("For a repetition within the recurrence, its period must be in units of days or weeks for a date-only alarm"));
+				mRecurrenceEdit->activateSubRepetition();   // display the alarm repetition dialog again
+				return;
+			}
 		}
 	}
 	if (checkText(mAlarmMessage))
@@ -1572,7 +1507,8 @@ void EditAlarmDlg::slotEditDeferral()
 	if (!mTimeWidget)
 		return;
 	bool limit = true;
-	int repeatCount = mSimpleRepetition->count();
+	int repeatInterval;
+	int repeatCount = mRecurrenceEdit->subRepeatCount(&repeatInterval);
 	DateTime start = mTimeWidget->getDateTime(0, !repeatCount, !mExpiredRecurrence);
 	if (!start.isValid())
 	{
@@ -1585,21 +1521,21 @@ void EditAlarmDlg::slotEditDeferral()
 	{
 		if (repeatCount  &&  start < now)
 		{
-			// Simple repetition - find the time of the next one
-			int interval = mSimpleRepetition->interval() * 60;
-			int repetition = (start.secsTo(now) + interval - 1) / interval;
+			// Sub-repetition - find the time of the next one
+			repeatInterval *= 60;
+			int repetition = (start.secsTo(now) + repeatInterval - 1) / repeatInterval;
 			if (repetition > repeatCount)
 			{
 				mTimeWidget->getDateTime();    // output the appropriate error message
 				return;
 			}
-			start = start.addSecs(repetition * interval);
+			start = start.addSecs(repetition * repeatInterval);
 		}
 	}
 
 	bool deferred = mDeferDateTime.isValid();
 	DeferAlarmDlg deferDlg(i18n("Defer Alarm"), (deferred ? mDeferDateTime : DateTime(now.addSecs(60))),
-	                       deferred, this, "deferDlg");
+	                       deferred, this, "EditDeferDlg");
 	if (limit)
 	{
 		// Don't allow deferral past the next recurrence
@@ -1672,16 +1608,17 @@ void EditAlarmDlg::slotShowRecurrenceEdit()
 /******************************************************************************
 *  Called when the recurrence type selection changes.
 *  Enables/disables date-only alarms as appropriate.
+*  Enables/disables controls depending on at-login setting.
 */
 void EditAlarmDlg::slotRecurTypeChange(int repeatType)
 {
+	bool atLogin = (mRecurrenceEdit->repeatType() == RecurrenceEdit::AT_LOGIN);
 	if (!mTemplate)
 	{
 		bool recurs = (mRecurrenceEdit->repeatType() != RecurrenceEdit::NO_RECUR);
 		if (mDeferGroup)
 			mDeferGroup->setEnabled(recurs);
 		mTimeWidget->enableAnyTime(!recurs || repeatType != RecurrenceEdit::SUBDAILY);
-		bool atLogin = (mRecurrenceEdit->repeatType() == RecurrenceEdit::AT_LOGIN);
 		if (atLogin)
 		{
 			mAlarmDateTime = mTimeWidget->getDateTime(0, false, false);
@@ -1689,47 +1626,38 @@ void EditAlarmDlg::slotRecurTypeChange(int repeatType)
 		}
 		mReminder->enableOnceOnly(recurs && !atLogin);
 	}
+	mReminder->setEnabled(!atLogin);
+        mLateCancel->setEnabled(!atLogin);
+        if (mShowInKorganizer)
+                mShowInKorganizer->setEnabled(!atLogin);
 	slotRecurFrequencyChange();
 }
 
 /******************************************************************************
-*  Called when the recurrence frequency selection changes, or the simple
+*  Called when the recurrence frequency selection changes, or the sub-
 *  repetition interval changes.
 *  Updates the recurrence frequency text.
 */
 void EditAlarmDlg::slotRecurFrequencyChange()
 {
+	slotSetSubRepetition();
 	KAEvent event;
 	mRecurrenceEdit->updateEvent(event, false);
-	event.setRepetition(mSimpleRepetition->interval(), mSimpleRepetition->count());
-	mRecurrenceText->setText(recurText(event));
+	mTabs->setTabLabel(mTabs->page(mRecurPageIndex), recurText(event));
 }
 
 /******************************************************************************
-*  Called when the Simple Repetition button has been pressed to display the
-*  alarm repetition dialog.
+*  Called when the Repetition within Recurrence button has been pressed to
+*  display the sub-repetition dialog.
 *  Alarm repetition has the following restrictions:
 *  1) Not allowed for a repeat-at-login alarm
 *  2) For a date-only alarm, the repeat interval must be a whole number of days.
 *  3) The overall repeat duration must be less than the recurrence interval.
 */
-void EditAlarmDlg::slotSetSimpleRepetition()
+void EditAlarmDlg::slotSetSubRepetition()
 {
 	bool dateOnly = mTemplate ? mTemplateAnyTime->isOn() : mTimeWidget->anyTime();
-	int maxDuration;
-	switch (mRecurrenceEdit->repeatType())
-	{
-		case RecurrenceEdit::NO_RECUR:  maxDuration = -1;  break;  // no restriction on repeat duration
-		case RecurrenceEdit::AT_LOGIN:  maxDuration = 0;  break;   // alarm repeat not allowed
-		default:          // repeat duration must be less than recurrence interval
-		{
-			KAEvent event;
-			mRecurrenceEdit->updateEvent(event, false);
-			maxDuration = event.longestRecurrenceInterval() - mReminder->minutes() - 1;
-			break;
-		}
-	}
-	mSimpleRepetition->initialise(mSimpleRepetition->interval(), mSimpleRepetition->count(), dateOnly, maxDuration);
+	mRecurrenceEdit->setSubRepetition(mReminder->minutes(), dateOnly);
 }
 
 /******************************************************************************
@@ -1832,10 +1760,13 @@ void EditAlarmDlg::slotAlarmTypeChanged(int)
 		mFilePadding->hide();
 		mTextMessageEdit->show();
 		mFontColourButton->show();
-		mReminder->show();
 		mSoundPicker->showSpeak(true);
+		mDisplayAlarmsFrame->show();
+		mCommandFrame->hide();
+		mEmailFrame->hide();
+		mReminder->show();
+		mConfirmAck->show();
 		setButtonWhatsThis(Try, i18n("Display the alarm message now"));
-		mAlarmTypeStack->raiseWidget(mDisplayAlarmsFrame);
 		focus = mTextMessageEdit;
 		displayAlarm = true;
 	}
@@ -1845,27 +1776,36 @@ void EditAlarmDlg::slotAlarmTypeChanged(int)
 		mFileBox->show();
 		mFilePadding->show();
 		mFontColourButton->hide();
-		mReminder->show();
 		mSoundPicker->showSpeak(false);
+		mDisplayAlarmsFrame->show();
+		mCommandFrame->hide();
+		mEmailFrame->hide();
+		mReminder->show();
+		mConfirmAck->show();
 		setButtonWhatsThis(Try, i18n("Display the file now"));
-		mAlarmTypeStack->raiseWidget(mDisplayAlarmsFrame);
 		mFileMessageEdit->setNoSelect();
 		focus = mFileMessageEdit;
 		displayAlarm = true;
 	}
 	else if (mCommandRadio->isOn())
 	{
+		mDisplayAlarmsFrame->hide();
+		mCommandFrame->show();
+		mEmailFrame->hide();
 		mReminder->hide();
+		mConfirmAck->hide();
 		setButtonWhatsThis(Try, i18n("Execute the specified command now"));
-		mAlarmTypeStack->raiseWidget(mCommandFrame);
 		mCmdCommandEdit->setNoSelect();
 		focus = mCmdCommandEdit;
 	}
 	else if (mEmailRadio->isOn())
 	{
+		mDisplayAlarmsFrame->hide();
+		mCommandFrame->hide();
+		mEmailFrame->show();
 		mReminder->hide();
+		mConfirmAck->hide();
 		setButtonWhatsThis(Try, i18n("Send the email to the specified addressees now"));
-		mAlarmTypeStack->raiseWidget(mEmailFrame);
 		mEmailToEdit->setNoSelect();
 		focus = mEmailToEdit;
 	}
@@ -1907,24 +1847,6 @@ void EditAlarmDlg::slotTemplateTimeType(int)
 	mTemplateTimeAfter->setEnabled(mTemplateUseTimeAfter->isOn());
 }
 
-/******************************************************************************
-*  Called when the a new background colour has been selected using the colour
-*  combo box.
-*/
-void EditAlarmDlg::slotBgColourSelected(const QColor& colour)
-{
-	mFontColourButton->setBgColour(colour);
-}
-
-/******************************************************************************
-*  Called when the a new font and colour have been selected using the font &
-*  colour pushbutton.
-*/
-void EditAlarmDlg::slotFontColourSelected()
-{
-	mBgColourChoose->setColour(mFontColourButton->bgColour());
-}
-
 /******************************************************************************
 *  Called when the "Any time" checkbox is toggled in the date/time widget.
 *  Sets the advance reminder and late cancel units to days if any time is checked.
@@ -2090,7 +2012,7 @@ bool EditAlarmDlg::checkText(QString& result, bool showErrorMessage) const
 = Provides KDE 2 compatibility.
 =============================================================================*/
 TextEdit::TextEdit(QWidget* parent, const char* name)
-	: QTextEdit(parent, name)
+	: KTextEdit(parent, name)
 {
 	QSize tsize = sizeHint();
 	tsize.setHeight(fontMetrics().lineSpacing()*13/4 + 2*frameWidth());
@@ -2101,5 +2023,5 @@ void TextEdit::dragEnterEvent(QDragEnterEvent* e)
 {
 	if (KCal::ICalDrag::canDecode(e))
 		e->accept(false);   // don't accept "text/calendar" objects
-	QTextEdit::dragEnterEvent(e);
+	KTextEdit::dragEnterEvent(e);
 }
