@@ -1,7 +1,7 @@
 /*
  *  kalarmiface.h  -  DCOP interface to KAlarm
  *  Program:  kalarm
- *  Copyright (C) 2004, 2005 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright © 2004-2006,2008 by David Jarvie <djarvie@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -115,8 +115,8 @@ class KAlarmIface : virtual public DCOPObject
 	 *  @param reminderMins   The number of minutes in advance of the main alarm and its recurrences to display
 	 *                        a reminder alarm, or 0 for no reminder.
 	 *  @param recurrence     Recurrence specification using iCalendar syntax (defined in RFC2445).
-	 *  @param repeatInterval Simple repetition repeat interval in minutes, or 0 for no simple repetition.
-	 *  @param repeatCount    Simple repetition repeat count (after the first occurrence), or 0 for no simple repetition.
+	 *  @param repeatInterval Simple repetition repeat interval in minutes, or 0 for no sub-repetition.
+	 *  @param repeatCount    Simple repetition repeat count (after the first occurrence), or 0 for no sub-repetition.
 	 *  @return true if alarm was scheduled successfully, false if configuration errors were found.
 	 */
 	virtual bool scheduleMessage(const QString& message, const QString& startDateTime, int lateCancel, unsigned flags,
@@ -190,8 +190,8 @@ class KAlarmIface : virtual public DCOPObject
 	 *  @param reminderMins   The number of minutes in advance of the main alarm and its recurrences to display
 	 *                        a reminder alarm, or 0 for no reminder.
 	 *  @param recurrence     Recurrence specification using iCalendar syntax (defined in RFC2445).
-	 *  @param repeatInterval Simple repetition repeat interval in minutes, or 0 for no simple repetition.
-	 *  @param repeatCount    Simple repetition repeat count (after the first occurrence), or 0 for no simple repetition.
+	 *  @param repeatInterval Simple repetition repeat interval in minutes, or 0 for no sub-repetition.
+	 *  @param repeatCount    Simple repetition repeat count (after the first occurrence), or 0 for no sub-repetition.
 	 *  @return true if alarm was scheduled successfully, false if configuration errors were found.
 	 */
 	virtual bool scheduleFile(const KURL& file, const QString& startDateTime, int lateCancel, unsigned flags, const QString& bgColor,
@@ -245,8 +245,8 @@ class KAlarmIface : virtual public DCOPObject
 	 *  @param lateCancel     Late-cancellation period in minutes, or 0 for no cancellation.
 	 *  @param flags          OR of flag bits defined in Flags enum.
 	 *  @param recurrence     Recurrence specification using iCalendar syntax (defined in RFC2445).
-	 *  @param repeatInterval Simple repetition repeat interval in minutes, or 0 for no simple repetition.
-	 *  @param repeatCount    Simple repetition repeat count (after the first occurrence), or 0 for no simple repetition.
+	 *  @param repeatInterval Simple repetition repeat interval in minutes, or 0 for no sub-repetition.
+	 *  @param repeatCount    Simple repetition repeat count (after the first occurrence), or 0 for no sub-repetition.
 	 *  @return true if alarm was scheduled successfully, false if configuration errors were found.
 	 */
 	virtual bool scheduleCommand(const QString& commandLine, const QString& startDateTime, int lateCancel, unsigned flags,
@@ -291,8 +291,8 @@ class KAlarmIface : virtual public DCOPObject
 	 *  @param lateCancel     Late-cancellation period in minutes, or 0 for no cancellation.
 	 *  @param flags          OR of flag bits defined in Flags enum.
 	 *  @param recurrence     Recurrence specification using iCalendar syntax (defined in RFC2445).
-	 *  @param repeatInterval Simple repetition repeat interval in minutes, or 0 for no simple repetition.
-	 *  @param repeatCount    Simple repetition repeat count (after the first occurrence), or 0 for no simple repetition.
+	 *  @param repeatInterval Simple repetition repeat interval in minutes, or 0 for no sub-repetition.
+	 *  @param repeatCount    Simple repetition repeat count (after the first occurrence), or 0 for no sub-repetition.
 	 *  @return true if alarm was scheduled successfully, false if configuration errors were found.
 	 */
 	virtual bool scheduleEmail(const QString& fromID, const QString& addresses, const QString& subject, const QString& message,
@@ -338,6 +338,18 @@ class KAlarmIface : virtual public DCOPObject
 	virtual bool scheduleEmail(const QString& fromID, const QString& addresses, const QString& subject, const QString& message,
 	                           const QString& attachments, const QString& startDateTime, int lateCancel, unsigned flags,
 	                           int repeatType, int repeatInterval, const QString& endDateTime) = 0;
+	/** Open the alarm edit dialog to edit an existing alarm.
+	 *  @param eventId       The unique ID of the event to be edited, or QString::null to create a new alarm.
+	 *  @return false if the alarm could not be found or is read-only, true otherwise.
+	 */
+	virtual bool edit(const QString& eventID) = 0;
+	/** Open the alarm edit dialog to edit a new alarm.
+	 *  @param templateName  Name of the alarm template to base the new alarm on, or QString::null if none.
+	 *                       If a template is specified but cannot be found, the alarm edit dialog is still
+	 *                       opened but is (obviously) not preset with the template.
+	 *  @return false if an alarm template was specified but could not be found, true otherwise.
+	 */
+	virtual bool editNew(const QString& templateName) = 0;
 };
 
 #endif // KALARMIFACE_H
