@@ -1,10 +1,10 @@
 /* -*- Mode: C++ -*-
-   $Id: KDGanttViewItem.h,v 1.8 2004/12/07 13:56:58 danders Exp $
+   $Id: KDGanttViewItem.h 766119 2008-01-25 10:19:31Z tilladam $
    KDGantt - a multi-platform charting engine
 */
 
 /****************************************************************************
- ** Copyright (C)  2002-2004 Klarlvdalens Datakonsult AB.  All rights reserved.
+ ** Copyright (C)  2002-2004 Klarälvdalens Datakonsult AB.  All rights reserved.
  **
  ** This file is part of the KDGantt library.
  **
@@ -185,6 +185,12 @@ public:
     static KDGanttViewItem* createFromDomElement( KDGanttViewItem* parent,
                                                   KDGanttViewItem* previous,
                                                   QDomElement& element );
+
+    void setMoveable( bool m );
+    bool isMoveable() const;
+    void setResizeable( bool r );
+    bool isResizeable() const;
+
 private:
     friend class KDGanttView;
     friend class KDTimeTableWidget;
@@ -239,6 +245,9 @@ private:
     bool shapeDefined;
     int _priority;
     static QDict<KDGanttViewItem> sItemDict;
+
+    bool _isMoveable;
+    bool _isResizeable;
 };
 
 
