@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     In addition, as a special exception, the copyright holders give
     permission to link the code of this program with any edition of
@@ -32,12 +32,14 @@
 #include <qapplication.h>
 #include <qlabel.h>
 #include <qcombobox.h>
+#include <qvaluevector.h>
 #include <qtimer.h>
 
 #include <kaction.h>
 #include <kiconloader.h>
 #include <klistview.h>
 #include <klocale.h>
+#include <ktoolbarbutton.h>
 
 #include "kmheaders.h"
 #include "kmsearchpattern.h"
@@ -49,7 +51,7 @@ HeaderListQuickSearch::HeaderListQuickSearch( QWidget *parent,
                                               KListView *listView,
                                               KActionCollection *actionCollection,
                                               const char *name )
-  : KListViewSearchLine(parent, listView, name), mStatusCombo(0), mStatus(0)
+  : KListViewSearchLine(parent, listView, name), mStatusCombo(0), mStatus(0),  statusList()
 {
   KAction *resetQuickSearch = new KAction( i18n( "Reset Quick Search" ),
                                            QApplication::reverseLayout()
@@ -68,8 +70,16 @@ HeaderListQuickSearch::HeaderListQuickSearch( QWidget *parent,
 
   mStatusCombo = new QComboBox( parent, "quick search status combo box" );
   mStatusCombo->insertItem( SmallIcon( "run" ), i18n("Any Status") );
-  for ( int i = 0; i < KMail::StatusValueCount; i++ )
-    mStatusCombo->insertItem( SmallIcon( KMail::StatusValues[ i ].icon ), i18n( KMail::StatusValues[ i ].text ) );
+
+  insertStatus( StatusUnread );
+  insertStatus( StatusNew );
+  insertStatus( StatusImportant );
+  insertStatus( StatusReplied );
+  insertStatus( StatusForwarded );
+  insertStatus( StatusToDo );
+  insertStatus( StatusHasAttachment );
+  insertStatus( StatusWatched );
+  insertStatus( StatusIgnored );
   mStatusCombo->setCurrentItem( 0 );
   mStatusCombo->installEventFilter( this );
   connect( mStatusCombo, SIGNAL ( activated( int ) ),
@@ -77,6 +87,10 @@ HeaderListQuickSearch::HeaderListQuickSearch( QWidget *parent,
 
   label->setBuddy( mStatusCombo );
 
+  KToolBarButton * btn = new KToolBarButton( "mail_find", 0, parent,
+                                            0, i18n( "Open Full Search" ) );
+  connect( btn, SIGNAL( clicked() ), SIGNAL( requestFullSearch() ) );
+
   /* Disable the signal connected by KListViewSearchLine since it will call 
    * itemAdded during KMHeaders::readSortOrder() which will in turn result
    * in getMsgBaseForItem( item ) wanting to access items which are no longer
@@ -128,6 +142,7 @@ bool HeaderListQuickSearch::eventFilter( QObject *watched, QEvent *event )
 
 bool HeaderListQuickSearch::itemMatches(const QListViewItem *item, const QString &s) const
 {
+  mCurrentSearchTerm = s; // bit of a hack, but works
   if ( mStatus != 0 ) {
     KMHeaders *headers = static_cast<KMHeaders*>( item->listView() );
     const KMMsgBase *msg = headers->getMsgBaseForItem( item );
@@ -150,11 +165,29 @@ void HeaderListQuickSearch::slotStatusChanged( int index )
   if ( index == 0 )
     mStatus = 0;
   else
-    mStatus =
-      KMSearchRuleStatus::statusFromEnglishName( KMail::StatusValues[ index - 1 ].text );
+    mStatus = KMSearchRuleStatus::statusFromEnglishName( statusList[index - 1] );
   updateSearch();
 }
 
+void HeaderListQuickSearch::insertStatus(KMail::StatusValueTypes which)
+{
+  mStatusCombo->insertItem( SmallIcon( KMail::StatusValues[which].icon ),
+    i18n( KMail::StatusValues[ which ].text ) );
+  statusList.append( KMail::StatusValues[ which ].text );
+}
+
+
+QString HeaderListQuickSearch::currentSearchTerm() const
+{
+    return mCurrentSearchTerm;
+}
+
+
+int HeaderListQuickSearch::currentStatus() const
+{
+    return mStatus;
+}
+
 } // namespace KMail
 
 #include "headerlistquicksearch.moc"
