@@ -54,6 +54,8 @@ using KMail::AccountManager;
 #include "kmacctcachedimap.h"
 #include "acljobs.h"
 
+#include "scalix.h"
+
 #include <mimelib/enum.h>
 #include <mimelib/utility.h>
 #include <mimelib/body.h>
@@ -126,6 +128,19 @@ const char* KMailICalIfaceImpl::annotationForContentsType( KMail::FolderContents
   return s_folderContentsType[type].annotation;
 }
 
+ExtraFolder::ExtraFolder( KMFolder* f )
+    : folder( f )
+{
+    folder->open("kmailicaliface::extrafolder");
+}
+
+ExtraFolder::~ExtraFolder()
+{
+    if ( folder )
+        folder->close("kmailicaliface::extrafolder");
+}
+
+
 /*
   This interface has three parts to it - libkcal interface;
   kmail interface; and helper functions.
@@ -192,6 +207,8 @@ static DwBodyPart* findBodyPart( const KMMessage& msg, const QString& attachment
     if ( part->hasHeaders()
          && attachmentName == part->Headers().ContentDisposition().Filename().c_str() )
       return part;
+    if ( part->hasHeaders() && attachmentName == part->Headers().ContentType().Name().c_str() )
+      return part;
   }
   return 0;
 }
@@ -333,17 +350,32 @@ bool KMailICalIfaceImpl::deleteAttachment( KMMessage& msg,
   return bOK;
 }
 
-static void setIcalVcardContentTypeHeader( KMMessage *msg, KMail::FolderContentsType t )
+static void setIcalVcardContentTypeHeader( KMMessage *msg, KMail::FolderContentsType t, KMFolder *folder )
 {
+  KMAcctCachedImap::GroupwareType groupwareType = KMAcctCachedImap::GroupwareKolab;
+
+  KMFolderCachedImap *imapFolder = dynamic_cast<KMFolderCachedImap*>( folder->storage() );
+  if ( imapFolder )
+    groupwareType = imapFolder->account()->groupwareType();
+
   msg->setType( DwMime::kTypeText );
   if ( t == KMail::ContentsTypeCalendar || t == KMail::ContentsTypeTask
       || t == KMail::ContentsTypeJournal ) {
     msg->setSubtype( DwMime::kSubtypeVCal );
-    msg->setHeaderField("Content-Type",
-        "text/calendar; method=REQUEST; charset=\"utf-8\"");
+
+    if ( groupwareType == KMAcctCachedImap::GroupwareKolab )
+      msg->setHeaderField("Content-Type",
+          "text/calendar; method=REQUEST; charset=\"utf-8\"");
+    else if ( groupwareType == KMAcctCachedImap::GroupwareScalix )
+      msg->setHeaderField("Content-Type",
+          "text/calendar; method=PUBLISH; charset=\"UTF-8\"");
+
   } else if ( t == KMail::ContentsTypeContact ) {
     msg->setSubtype( DwMime::kSubtypeXVCard );
-    msg->setHeaderField( "Content-Type", "Text/X-VCard; charset=\"utf-8\"" );
+    if ( groupwareType == KMAcctCachedImap::GroupwareKolab )
+      msg->setHeaderField( "Content-Type", "Text/X-VCard; charset=\"utf-8\"" );
+    else if ( groupwareType == KMAcctCachedImap::GroupwareScalix )
+      msg->setHeaderField( "Content-Type", "application/scalix-properties; charset=\"UTF-8\"" );
   } else {
     kdWarning(5006) << k_funcinfo << "Attempt to write non-groupware contents to folder" << endl;
   }
@@ -397,7 +429,7 @@ Q_UINT32 KMailICalIfaceImpl::addIncidenceKolab( KMFolder& folder,
     setXMLContentTypeHeader( msg, plainTextBody );
   } else if ( storageFormat( &folder ) == StorageIcalVcard ) {
     const KMail::FolderContentsType t = folder.storage()->contentsType();
-    setIcalVcardContentTypeHeader( msg, t );
+    setIcalVcardContentTypeHeader( msg, t, &folder );
     msg->setBodyEncoded( plainTextBody.utf8() );
   } else {
     kdWarning(5006) << k_funcinfo << "Attempt to write to folder with unknown storage type" << endl;
@@ -434,6 +466,7 @@ Q_UINT32 KMailICalIfaceImpl::addIncidenceKolab( KMFolder& folder,
 
     //debugBodyParts( "after addMsg", *msg );
     addFolderChange( &folder, Contents );
+    syncFolder( &folder );
   } else
     kdError(5006) << "addIncidenceKolab(): Message *NOT* saved!\n";
 
@@ -463,6 +496,7 @@ bool KMailICalIfaceImpl::deleteIncidenceKolab( const QString& resource,
   if( msg ) {
     // Message found - delete it and return happy
     deleteMsg( msg );
+    syncFolder( f );
     rc = true;
   } else {
     kdDebug(5006) << "Message not found, cannot remove serNum " << sernum << endl;
@@ -512,7 +546,7 @@ QMap<Q_UINT32, QString> KMailICalIfaceImpl::incidencesKolab( const QString& mime
     return aMap;
   }
 
-  f->open("incidences");
+  f->open( "incidences" );
 
   int stopIndex = nbMessages == -1 ? f->count() :
                   QMIN( f->count(), startIndex + nbMessages );
@@ -605,6 +639,58 @@ void KMailICalIfaceImpl::slotMessageRetrieved( KMMessage* msg )
   }
 }
 
+static int dimapAccountCount()
+{
+  KMail::AccountManager *mgr = kmkernel->acctMgr();
+  KMAccount *account = mgr->first();
+  int count = 0;
+  while ( account ) {
+    if ( dynamic_cast<KMAcctCachedImap*>( account ) )
+      ++count;
+    account = mgr->next();
+  }
+  return count;
+}
+
+static QString subresourceLabelForPresentation( const KMFolder * folder )
+{
+    QString label = folder->prettyURL();
+    QStringList parts = QStringList::split( QString::fromLatin1("/"), label );
+    // In the common special case of some other user's folder shared with us
+    // the url looks like "Server Name/user/$USERNAME/Folder/Name". Make
+    // those a bit nicer.
+    if ( parts[1] == QString::fromLatin1("user") ) {
+        QStringList remainder(parts);
+        remainder.pop_front();
+        remainder.pop_front();
+        remainder.pop_front();
+        label = i18n("%1's %2")
+            .arg( parts[2] )
+            .arg( remainder.join( QString::fromLatin1("/") ) );
+    }
+    // Another special case is our own folders, under the imap INBOX, make
+    // those prettier too
+    const KMFolder *parent = folder;
+    while ( parent->parent() && parent->parent()->owner() ) {
+      parent = parent->parent()->owner();
+      if ( parent->isSystemFolder() ) {
+        QStringList remainder(parts);
+        remainder.pop_front();
+        remainder.pop_front();
+        if ( dimapAccountCount() > 1 ) {
+          label = i18n( "My %1 (%2)")
+              .arg( remainder.join( QString::fromLatin1("/") ),
+                    static_cast<const KMFolderCachedImap*>( folder->storage() )->account()->name() );
+        } else {
+          label = i18n("My %1")
+              .arg( remainder.join( QString::fromLatin1("/") ) );
+        }
+        break;
+      }
+    }
+    return label;
+}
+
 /* list all available subresources */
 QValueList<KMailICalIfaceImpl::SubResource> KMailICalIfaceImpl::subresourcesKolab( const QString& contentsType )
 {
@@ -613,7 +699,7 @@ QValueList<KMailICalIfaceImpl::SubResource> KMailICalIfaceImpl::subresourcesKola
   // Add the default one
   KMFolder* f = folderFromType( contentsType, QString::null );
   if ( f ) {
-    subResources.append( SubResource( f->location(),  f->prettyURL(),
+    subResources.append( SubResource( f->location(), subresourceLabelForPresentation( f ),
                                       !f->isReadOnly(), folderIsAlarmRelevant( f ) ) );
     kdDebug(5006) << "Adding(1) folder " << f->location() << "    " <<
       ( f->isReadOnly() ? "readonly" : "" ) << endl;
@@ -625,7 +711,7 @@ QValueList<KMailICalIfaceImpl::SubResource> KMailICalIfaceImpl::subresourcesKola
   for ( ; it.current(); ++it ){
     f = it.current()->folder;
     if ( f && f->storage()->contentsType() == t ) {
-      subResources.append( SubResource( f->location(), f->prettyURL(),
+      subResources.append( SubResource( f->location(), subresourceLabelForPresentation( f ),
                                         !f->isReadOnly(), folderIsAlarmRelevant( f ) ) );
       kdDebug(5006) << "Adding(2) folder " << f->location() << "     " <<
               ( f->isReadOnly() ? "readonly" : "" ) << endl;
@@ -731,7 +817,7 @@ Q_UINT32 KMailICalIfaceImpl::update( const QString& resource,
     return rc;
   }
 
-  f->open("ifaceupdate");
+  f->open( "ifaceupdate" );
 
   KMMessage* msg = 0;
   if ( sernum != 0 ) {
@@ -765,7 +851,7 @@ Q_UINT32 KMailICalIfaceImpl::update( const QString& resource,
     if ( storageFormat( f ) == StorageIcalVcard ) {
       //kdDebug(5006) << k_funcinfo << " StorageFormatIcalVcard " << endl;
       if ( !messageWasIcalVcardFormat ) {
-        setIcalVcardContentTypeHeader( newMsg, t );
+        setIcalVcardContentTypeHeader( newMsg, t, f );
       }
       newMsg->setBodyEncoded( plainTextBody.utf8() );
     } else if ( storageFormat( f ) == StorageXML ) {
@@ -806,6 +892,7 @@ Q_UINT32 KMailICalIfaceImpl::update( const QString& resource,
       kdDebug(5006) << "forget about " << sernum << ", it's " << rc << " now" << endl;
     }
     addFolderChange( f, Contents );
+    syncFolder( f );
   } else {
     // Message not found - store it newly
     rc = addIncidenceKolab( *f, subject, plainTextBody, customHeaders,
@@ -878,6 +965,78 @@ KURL KMailICalIfaceImpl::getAttachment( const QString& resource,
   return url;
 }
 
+QString KMailICalIfaceImpl::attachmentMimetype( const QString & resource,
+                                                Q_UINT32 sernum,
+                                                const QString & filename )
+{
+  if( !mUseResourceIMAP )
+    return QString();
+  KMFolder* f = findResourceFolder( resource );
+  if( !f || storageFormat( f ) != StorageXML ) {
+    kdError(5006) << "attachmentMimetype(" << resource << ") : Wrong folder" << endl;
+    return QString();
+  }
+
+  KMMessage* msg = findMessageBySerNum( sernum, f );
+  if( msg ) {
+    // Message found - look for the attachment:
+    DwBodyPart* part = findBodyPart( *msg, filename );
+    if ( part ) {
+      KMMessagePart kmPart;
+      msg->bodyPart( part, &kmPart );
+      return QString( kmPart.typeStr() ) + "/" + QString( kmPart.subtypeStr() );
+    } else {
+      kdDebug(5006) << "Attachment " << filename << " not found." << endl;
+    }
+  } else {
+    kdDebug(5006) << "Message not found." << endl;
+  }
+
+  return QString();
+}
+
+QStringList KMailICalIfaceImpl::listAttachments(const QString & resource, Q_UINT32 sernum)
+{
+  QStringList rv;
+  if( !mUseResourceIMAP )
+    return rv;
+
+  // Find the folder
+  KMFolder* f = findResourceFolder( resource );
+  if( !f ) {
+    kdError(5006) << "listAttachments(" << resource << ") : Not an IMAP resource folder" << endl;
+    return rv;
+  }
+  if ( storageFormat( f ) != StorageXML ) {
+    kdError(5006) << "listAttachment(" << resource << ") : Folder has wrong storage format " << storageFormat( f ) << endl;
+    return rv;
+  }
+
+  KMMessage* msg = findMessageBySerNum( sernum, f );
+  if( msg ) {
+    for ( DwBodyPart* part = msg->getFirstDwBodyPart(); part; part = part->Next() ) {
+      if ( part->hasHeaders() ) {
+        QString name;
+        DwMediaType& contentType = part->Headers().ContentType();
+        if ( QString( contentType.SubtypeStr().c_str() ).startsWith( "x-vnd.kolab." )
+           || QString( contentType.SubtypeStr().c_str() ).contains( "tnef" ) )
+          continue;
+        if ( !part->Headers().ContentDisposition().Filename().empty() )
+          name = part->Headers().ContentDisposition().Filename().c_str();
+        else if ( !contentType.Name().empty() )
+          name = contentType.Name().c_str();
+        if ( !name.isEmpty() )
+          rv.append( name );
+      }
+    }
+  } else {
+    kdDebug(5006) << "Message not found." << endl;
+  }
+
+  return rv;
+}
+
+
 // ============================================================================
 
 /* KMail part of the interface. These slots are connected to the resource
@@ -1090,7 +1249,7 @@ bool KMailICalIfaceImpl::hideResourceFolder( KMFolder* folder ) const
 bool KMailICalIfaceImpl::hideResourceAccountRoot( KMFolder* folder ) const
 {
   KMFolderCachedImap *dimapFolder = dynamic_cast<KMFolderCachedImap*>( folder->storage() );
-  bool hide = dimapFolder && mHideFolders 
+  bool hide = dimapFolder && mHideFolders
        && (int)dimapFolder->account()->id() == GlobalSettings::self()->theIMAPResourceAccount()
        && GlobalSettings::self()->showOnlyGroupwareFoldersForGroupwareAccount();
   return hide;
@@ -1184,6 +1343,7 @@ KMMessage *KMailICalIfaceImpl::findMessageBySerNum( Q_UINT32 serNum, KMFolder* f
   KMFolder* aFolder = 0;
   int index;
   KMMsgDict::instance()->getLocation( serNum, &aFolder, &index );
+
   if( aFolder && aFolder != folder ) {
     kdWarning(5006) << "findMessageBySerNum( " << serNum << " ) found it in folder " << aFolder->location() << ", expected " << folder->location() << endl;
   } else {
@@ -1203,8 +1363,15 @@ void KMailICalIfaceImpl::deleteMsg( KMMessage *msg )
   KMFolder *srcFolder = msg->parent();
   int idx = srcFolder->find(msg);
   assert(idx != -1);
-  srcFolder->removeMsg(idx);
-  delete msg;
+  // kill existing jobs since we are about to delete the message
+  srcFolder->ignoreJobsForMessage( msg );
+  if ( !msg->transferInProgress() ) {
+    srcFolder->removeMsg(idx);
+    delete msg;
+  } else {
+    kdDebug(5006) << k_funcinfo << "Message cannot be deleted now because it is currently in use " << msg << endl;
+    msg->deleteWhenUnused();
+  }
   addFolderChange( srcFolder, Contents );
 }
 
@@ -1263,7 +1430,7 @@ void KMailICalIfaceImpl::folderContentsTypeChanged( KMFolder* folder,
     connectFolder( folder );
   }
   // Tell about the new resource
-  subresourceAdded( folderContentsType( contentsType ), location, folder->prettyURL(),
+  subresourceAdded( folderContentsType( contentsType ), location, subresourceLabelForPresentation(folder),
                     !folder->isReadOnly(), folderIsAlarmRelevant( folder ) );
 }
 
@@ -1408,7 +1575,7 @@ void KMailICalIfaceImpl::slotFolderPropertiesChanged( KMFolder* folder )
     const QString contentsTypeStr = folderContentsType( folder->storage()->contentsType() );
     subresourceDeleted( contentsTypeStr, location );
 
-    subresourceAdded( contentsTypeStr, location, folder->prettyURL(),
+    subresourceAdded( contentsTypeStr, location, subresourceLabelForPresentation( folder ),
                       !folder->isReadOnly(), folderIsAlarmRelevant( folder ) );
 
   }
@@ -1516,145 +1683,223 @@ void KMailICalIfaceImpl::readConfig()
     folderType = folderParent->folderType();
   }
 
-  // Make sure the folder parent has the subdirs
-  // Globally there are 3 cases: nothing found, some stuff found by type/name heuristics, or everything found OK
-  bool noneFound = true;
-  bool mustFix = false; // true when at least one was found by heuristics
-  QValueVector<StandardFolderSearchResult> results( KMail::ContentsTypeLast + 1 );
-  for ( int i = 0; i < KMail::ContentsTypeLast+1; ++i ) {
-    if ( i != KMail::ContentsTypeMail ) {
-      results[i] = findStandardResourceFolder( folderParentDir, static_cast<KMail::FolderContentsType>(i) );
-      if ( results[i].found == StandardFolderSearchResult::FoundAndStandard )
-        noneFound = false;
-      else if ( results[i].found == StandardFolderSearchResult::FoundByType ||
-                results[i].found == StandardFolderSearchResult::FoundByName ) {
-        mustFix = true;
-        noneFound = false;
-      } else // NotFound
-        mustFix = true;
-    }
-  }
+  KMAcctCachedImap::GroupwareType groupwareType = dynamic_cast<KMFolderCachedImap *>( folderParent->storage() )->account()->groupwareType();
 
-  // Check if something changed
-  if( mUseResourceIMAP && !noneFound && !mustFix && mFolderParentDir == folderParentDir
-      && mFolderType == folderType ) {
-    // Nothing changed
-    if ( hideFolders != mHideFolders ) {
-      // Well, the folder hiding has changed
-      mHideFolders = hideFolders;
-      reloadFolderTree();
-    }
-    return;
-  }
-
-  if( noneFound || mustFix ) {
-    QString msg;
-    QString parentFolderName = folderParent != 0 ? folderParent->name() : folderParentDir->name();
-    if ( noneFound ) {
-      // No subfolder was found, so ask if we can make them
-      msg = i18n("KMail will now create the required groupware folders"
-                 " as subfolders of %1; if you do not want this, cancel"
-                 " and the IMAP resource will be disabled").arg(parentFolderName);
-    } else {
-      // Some subfolders were found, be more precise
-      QString operations = "<ul>";
-      for ( int i = 0; i < KMail::ContentsTypeLast+1; ++i ) {
-        if ( i != KMail::ContentsTypeMail ) {
-          QString typeName = localizedDefaultFolderName( static_cast<KMail::FolderContentsType>( i ) );
-          if ( results[i].found == StandardFolderSearchResult::NotFound )
-            operations += "<li>" + i18n( "%1: no folder found. It will be created." ).arg( typeName ) + "</li>";
-          else if ( results[i].found == StandardFolderSearchResult::FoundByType || results[i].found == StandardFolderSearchResult::FoundByName )
-            operations += "<li>" + i18n( "%1: found folder %2. It will be set as the main groupware folder." ).
-                          arg( typeName ).arg( results[i].folder->label() ) + "</li>";
-        }
+  if ( groupwareType == KMAcctCachedImap::GroupwareKolab ) {
+    // Make sure the folder parent has the subdirs
+    // Globally there are 3 cases: nothing found, some stuff found by type/name heuristics, or everything found OK
+    bool noneFound = true;
+    bool mustFix = false; // true when at least one was found by heuristics
+    QValueVector<StandardFolderSearchResult> results( KMail::ContentsTypeLast + 1 );
+    for ( int i = 0; i < KMail::ContentsTypeLast+1; ++i ) {
+      if ( i != KMail::ContentsTypeMail ) {
+        results[i] = findStandardResourceFolder( folderParentDir, static_cast<KMail::FolderContentsType>(i) );
+        if ( results[i].found == StandardFolderSearchResult::FoundAndStandard )
+          noneFound = false;
+        else if ( results[i].found == StandardFolderSearchResult::FoundByType ||
+                  results[i].found == StandardFolderSearchResult::FoundByName ) {
+          mustFix = true;
+          noneFound = false;
+        } else // NotFound
+          mustFix = true;
       }
-      operations += "</ul>";
-
-      msg = i18n("<qt>KMail found the following groupware folders in %1 and needs to perform the following operations: %2"
-                 "<br>If you do not want this, cancel"
-                 " and the IMAP resource will be disabled").arg(parentFolderName, operations);
-
     }
 
-    if( KMessageBox::questionYesNo( 0, msg,
-                                    i18n("Standard Groupware Folders"), KStdGuiItem::cont(), KStdGuiItem::cancel() ) == KMessageBox::No ) {
-
-      GlobalSettings::self()->setTheIMAPResourceEnabled( false );
-      mUseResourceIMAP = false;
-      mFolderParentDir = 0;
-      mFolderParent = 0;
-      reloadFolderTree();
+    // Check if something changed
+    if( mUseResourceIMAP && !noneFound && !mustFix && mFolderParentDir == folderParentDir
+        && mFolderType == folderType ) {
+      // Nothing changed
+      if ( hideFolders != mHideFolders ) {
+        // Well, the folder hiding has changed
+        mHideFolders = hideFolders;
+        reloadFolderTree();
+      }
       return;
     }
-  }
 
-  // Make the new settings work
-  mUseResourceIMAP = true;
-  mFolderLanguage = GlobalSettings::self()->theIMAPResourceFolderLanguage();
-  if( mFolderLanguage > 3 ) mFolderLanguage = 0;
-  mFolderParentDir = folderParentDir;
-  mFolderParent = folderParent;
-  mFolderType = folderType;
-  mHideFolders = hideFolders;
+    if( noneFound || mustFix ) {
+      QString msg;
+      QString parentFolderName = folderParent != 0 ? folderParent->name() : folderParentDir->name();
+      if ( noneFound ) {
+        // No subfolder was found, so ask if we can make them
+        msg = i18n("KMail will now create the required groupware folders"
+                   " as subfolders of %1; if you do not want this, cancel"
+                   " and the IMAP resource will be disabled").arg(parentFolderName);
+      } else {
+        // Some subfolders were found, be more precise
+        QString operations = "<ul>";
+        for ( int i = 0; i < KMail::ContentsTypeLast+1; ++i ) {
+          if ( i != KMail::ContentsTypeMail ) {
+            QString typeName = localizedDefaultFolderName( static_cast<KMail::FolderContentsType>( i ) );
+            if ( results[i].found == StandardFolderSearchResult::NotFound )
+              operations += "<li>" + i18n( "%1: no folder found. It will be created." ).arg( typeName ) + "</li>";
+            else if ( results[i].found == StandardFolderSearchResult::FoundByType || results[i].found == StandardFolderSearchResult::FoundByName )
+              operations += "<li>" + i18n( "%1: found folder %2. It will be set as the main groupware folder." ).
+                            arg( typeName ).arg( results[i].folder->label() ) + "</li>";
+          }
+        }
+        operations += "</ul>";
 
-  // Close the previous folders
-  cleanup();
+        msg = i18n("<qt>KMail found the following groupware folders in %1 and needs to perform the following operations: %2"
+                   "<br>If you do not want this, cancel"
+                   " and the IMAP resource will be disabled").arg(parentFolderName, operations);
 
-  // Set the new folders
-  mCalendar = initFolder( KMail::ContentsTypeCalendar );
-  mTasks    = initFolder( KMail::ContentsTypeTask );
-  mJournals = initFolder( KMail::ContentsTypeJournal );
-  mContacts = initFolder( KMail::ContentsTypeContact );
-  mNotes    = initFolder( KMail::ContentsTypeNote );
+      }
 
-  // Store final annotation (with .default) so that we won't ask again on next startup
-  if ( mCalendar->folderType() == KMFolderTypeCachedImap )
-    static_cast<KMFolderCachedImap *>( mCalendar->storage() )->updateAnnotationFolderType();
-  if ( mTasks->folderType() == KMFolderTypeCachedImap )
-    static_cast<KMFolderCachedImap *>( mTasks->storage() )->updateAnnotationFolderType();
-  if ( mJournals->folderType() == KMFolderTypeCachedImap )
-    static_cast<KMFolderCachedImap *>( mJournals->storage() )->updateAnnotationFolderType();
-  if ( mContacts->folderType() == KMFolderTypeCachedImap )
-    static_cast<KMFolderCachedImap *>( mContacts->storage() )->updateAnnotationFolderType();
-  if ( mNotes->folderType() == KMFolderTypeCachedImap )
-    static_cast<KMFolderCachedImap *>( mNotes->storage() )->updateAnnotationFolderType();
+      if( KMessageBox::questionYesNo( 0, msg,
+                                      i18n("Standard Groupware Folders"), KStdGuiItem::cont(), KStdGuiItem::cancel() ) == KMessageBox::No ) {
 
-  // BEGIN TILL TODO The below only uses the dimap folder manager, which
-  // will fail for all other folder types. Adjust.
-
-  kdDebug(5006) << k_funcinfo << "mCalendar=" << mCalendar << " " << mCalendar->location() << endl;
-  kdDebug(5006) << k_funcinfo << "mContacts=" << mContacts << " " << mContacts->location() << endl;
-  kdDebug(5006) << k_funcinfo << "mNotes=" << mNotes << " " << mNotes->location() << endl;
-
-  // Find all extra folders
-  QStringList folderNames;
-  QValueList<QGuardedPtr<KMFolder> > folderList;
-  kmkernel->dimapFolderMgr()->createFolderList(&folderNames, &folderList);
-  for(QValueList<QGuardedPtr<KMFolder> >::iterator it = folderList.begin();
-      it != folderList.end(); ++it)
-  {
-    FolderStorage* storage = (*it)->storage();
-    if ( storage->contentsType() != 0 ) {
-      folderContentsTypeChanged( *it, storage->contentsType() );
+        GlobalSettings::self()->setTheIMAPResourceEnabled( false );
+        mUseResourceIMAP = false;
+        mFolderParentDir = 0;
+        mFolderParent = 0;
+        reloadFolderTree();
+        return;
+      }
     }
+
+    // Make the new settings work
+    mUseResourceIMAP = true;
+    mFolderLanguage = GlobalSettings::self()->theIMAPResourceFolderLanguage();
+    if( mFolderLanguage > 3 ) mFolderLanguage = 0;
+    mFolderParentDir = folderParentDir;
+    mFolderParent = folderParent;
+    mFolderType = folderType;
+    mHideFolders = hideFolders;
+
+    // Close the previous folders
+    cleanup();
+
+    // Set the new folders
+    mCalendar = initFolder( KMail::ContentsTypeCalendar );
+    mTasks    = initFolder( KMail::ContentsTypeTask );
+    mJournals = initFolder( KMail::ContentsTypeJournal );
+    mContacts = initFolder( KMail::ContentsTypeContact );
+    mNotes    = initFolder( KMail::ContentsTypeNote );
+
+    // Store final annotation (with .default) so that we won't ask again on next startup
+    if ( mCalendar->folderType() == KMFolderTypeCachedImap )
+      static_cast<KMFolderCachedImap *>( mCalendar->storage() )->updateAnnotationFolderType();
+    if ( mTasks->folderType() == KMFolderTypeCachedImap )
+      static_cast<KMFolderCachedImap *>( mTasks->storage() )->updateAnnotationFolderType();
+    if ( mJournals->folderType() == KMFolderTypeCachedImap )
+      static_cast<KMFolderCachedImap *>( mJournals->storage() )->updateAnnotationFolderType();
+    if ( mContacts->folderType() == KMFolderTypeCachedImap )
+      static_cast<KMFolderCachedImap *>( mContacts->storage() )->updateAnnotationFolderType();
+    if ( mNotes->folderType() == KMFolderTypeCachedImap )
+      static_cast<KMFolderCachedImap *>( mNotes->storage() )->updateAnnotationFolderType();
+
+    // BEGIN TILL TODO The below only uses the dimap folder manager, which
+    // will fail for all other folder types. Adjust.
+
+    kdDebug(5006) << k_funcinfo << "mCalendar=" << mCalendar << " " << mCalendar->location() << endl;
+    kdDebug(5006) << k_funcinfo << "mContacts=" << mContacts << " " << mContacts->location() << endl;
+    kdDebug(5006) << k_funcinfo << "mNotes=" << mNotes << " " << mNotes->location() << endl;
+
+    // Find all extra folders
+    QStringList folderNames;
+    QValueList<QGuardedPtr<KMFolder> > folderList;
+    kmkernel->dimapFolderMgr()->createFolderList(&folderNames, &folderList);
+    for(QValueList<QGuardedPtr<KMFolder> >::iterator it = folderList.begin();
+        it != folderList.end(); ++it)
+    {
+      FolderStorage* storage = (*it)->storage();
+      if ( storage->contentsType() != 0 ) {
+        folderContentsTypeChanged( *it, storage->contentsType() );
+      }
+    }
+
+    // If we just created them, they might have been registered as extra folders temporarily.
+    // -> undo that.
+    mExtraFolders.remove( mCalendar->location() );
+    mExtraFolders.remove( mTasks->location() );
+    mExtraFolders.remove( mJournals->location() );
+    mExtraFolders.remove( mContacts->location() );
+    mExtraFolders.remove( mNotes->location() );
+
+    // END TILL TODO
+
+    subresourceAdded( folderContentsType( KMail::ContentsTypeCalendar ), mCalendar->location(), mCalendar->label(), true, true );
+    subresourceAdded( folderContentsType( KMail::ContentsTypeTask ), mTasks->location(), mTasks->label(), true, true );
+    subresourceAdded( folderContentsType( KMail::ContentsTypeJournal ), mJournals->location(), mJournals->label(), true, false );
+    subresourceAdded( folderContentsType( KMail::ContentsTypeContact ), mContacts->location(), mContacts->label(), true, false );
+    subresourceAdded( folderContentsType( KMail::ContentsTypeNote ), mNotes->location(), mNotes->label(), true, false );
+  } else if ( groupwareType == KMAcctCachedImap::GroupwareScalix ) {
+    // Make the new settings work
+    mUseResourceIMAP = true;
+    mFolderParentDir = folderParentDir;
+    mFolderParent = folderParent;
+    mFolderType = folderType;
+    mHideFolders = false;
+
+    // Close the previous folders
+    cleanup();
+
+    // Set the new folders
+    mCalendar = initScalixFolder( KMail::ContentsTypeCalendar );
+    mTasks    = initScalixFolder( KMail::ContentsTypeTask );
+    mJournals = 0;
+    mContacts = initScalixFolder( KMail::ContentsTypeContact );
+    mNotes    = initScalixFolder( KMail::ContentsTypeNote );
+
+    // Store final annotation (with .default) so that we won't ask again on next startup
+    if ( mCalendar->folderType() == KMFolderTypeCachedImap )
+      static_cast<KMFolderCachedImap *>( mCalendar->storage() )->updateAnnotationFolderType();
+    if ( mTasks->folderType() == KMFolderTypeCachedImap )
+      static_cast<KMFolderCachedImap *>( mTasks->storage() )->updateAnnotationFolderType();
+    if ( mContacts->folderType() == KMFolderTypeCachedImap )
+      static_cast<KMFolderCachedImap *>( mContacts->storage() )->updateAnnotationFolderType();
+    if ( mNotes->folderType() == KMFolderTypeCachedImap )
+      static_cast<KMFolderCachedImap *>( mNotes->storage() )->updateAnnotationFolderType();
+
+    // BEGIN TILL TODO The below only uses the dimap folder manager, which
+    // will fail for all other folder types. Adjust.
+
+    kdDebug(5006) << k_funcinfo << "mCalendar=" << mCalendar << " " << mCalendar->location() << endl;
+    kdDebug(5006) << k_funcinfo << "mContacts=" << mContacts << " " << mContacts->location() << endl;
+    kdDebug(5006) << k_funcinfo << "mNotes=" << mNotes << " " << mNotes->location() << endl;
+
+    // Find all extra folders
+    QStringList folderNames;
+    QValueList<QGuardedPtr<KMFolder> > folderList;
+    kmkernel->dimapFolderMgr()->createFolderList(&folderNames, &folderList);
+    QValueList<QGuardedPtr<KMFolder> >::iterator it;
+    for(it = folderList.begin(); it != folderList.end(); ++it)
+    {
+      FolderStorage *storage = (*it)->storage();
+
+      if ( (*it)->folderType() == KMFolderTypeCachedImap ) {
+        KMFolderCachedImap *imapFolder = static_cast<KMFolderCachedImap*>( storage );
+
+        const QString attributes = imapFolder->folderAttributes();
+        if ( attributes.contains( "X-FolderClass" ) ) {
+          if ( !attributes.contains( "X-SpecialFolder" ) || (*it)->location().contains( "@" ) ) {
+            const Scalix::FolderAttributeParser parser( attributes );
+            if ( !parser.folderClass().isEmpty() ) {
+              FolderContentsType type = Scalix::Utils::scalixIdToContentsType( parser.folderClass() );
+              imapFolder->setContentsType( type );
+              folderContentsTypeChanged( *it, type );
+            }
+          }
+        }
+      }
+    }
+
+    // If we just created them, they might have been registered as extra folders temporarily.
+    // -> undo that.
+    mExtraFolders.remove( mCalendar->location() );
+    mExtraFolders.remove( mTasks->location() );
+    mExtraFolders.remove( mContacts->location() );
+    mExtraFolders.remove( mNotes->location() );
+
+    // END TILL TODO
+
+    subresourceAdded( folderContentsType( KMail::ContentsTypeCalendar ), mCalendar->location(), mCalendar->label(), true, true );
+    subresourceAdded( folderContentsType( KMail::ContentsTypeTask ), mTasks->location(), mTasks->label(), true, true );
+    subresourceAdded( folderContentsType( KMail::ContentsTypeContact ), mContacts->location(), mContacts->label(), true, false );
+    subresourceAdded( folderContentsType( KMail::ContentsTypeNote ), mNotes->location(), mNotes->label(), true, false );
   }
 
-  // If we just created them, they might have been registered as extra folders temporarily.
-  // -> undo that.
-  mExtraFolders.remove( mCalendar->location() );
-  mExtraFolders.remove( mTasks->location() );
-  mExtraFolders.remove( mJournals->location() );
-  mExtraFolders.remove( mContacts->location() );
-  mExtraFolders.remove( mNotes->location() );
-
-  // END TILL TODO
-
-  subresourceAdded( folderContentsType( KMail::ContentsTypeCalendar ), mCalendar->location(), mCalendar->label(), true, true );
-  subresourceAdded( folderContentsType( KMail::ContentsTypeTask ), mTasks->location(), mTasks->label(), true, true );
-  subresourceAdded( folderContentsType( KMail::ContentsTypeJournal ), mJournals->location(), mJournals->label(), true, false );
-  subresourceAdded( folderContentsType( KMail::ContentsTypeContact ), mContacts->location(), mContacts->label(), true, false );
-  subresourceAdded( folderContentsType( KMail::ContentsTypeNote ), mNotes->location(), mNotes->label(), true, false );
-
   reloadFolderTree();
 }
 
@@ -1716,6 +1961,59 @@ KMFolder* KMailICalIfaceImpl::initFolder( KMail::FolderContentsType contentsType
   return folder;
 }
 
+KMFolder* KMailICalIfaceImpl::initScalixFolder( KMail::FolderContentsType contentsType )
+{
+  // Figure out what type of folder this is supposed to be
+  KMFolderType type = mFolderType;
+  if( type == KMFolderTypeUnknown ) type = KMFolderTypeMaildir;
+
+  KMFolder* folder = 0;
+
+  // Find all extra folders
+  QStringList folderNames;
+  QValueList<QGuardedPtr<KMFolder> > folderList;
+  Q_ASSERT( kmkernel );
+  Q_ASSERT( kmkernel->dimapFolderMgr() );
+  kmkernel->dimapFolderMgr()->createFolderList(&folderNames, &folderList);
+  QValueList<QGuardedPtr<KMFolder> >::iterator it = folderList.begin();
+  for(; it != folderList.end(); ++it)
+  {
+    FolderStorage *storage = (*it)->storage();
+
+    if ( (*it)->folderType() == KMFolderTypeCachedImap ) {
+      KMFolderCachedImap *imapFolder = static_cast<KMFolderCachedImap*>( storage );
+
+      const QString attributes = imapFolder->folderAttributes();
+      if ( attributes.contains( "X-SpecialFolder" ) ) {
+        const Scalix::FolderAttributeParser parser( attributes );
+        if ( contentsType == Scalix::Utils::scalixIdToContentsType( parser.folderClass() ) ) {
+          folder = *it;
+          break;
+        }
+      }
+    }
+  }
+
+  if ( !folder ) {
+    return 0;
+  } else {
+    FolderInfo info = readFolderInfo( folder );
+    mFolderInfoMap.insert( folder, info );
+    //kdDebug(5006) << "Found existing folder type " << itemType << " : " << folder->location()  << endl;
+  }
+
+  if( folder->canAccess() != 0 ) {
+    KMessageBox::sorry(0, i18n("You do not have read/write permission to your folder.") );
+    return 0;
+  }
+  folder->storage()->setContentsType( contentsType );
+  folder->setSystemFolder( true );
+  folder->storage()->writeConfig();
+  folder->open( "scalixfolder" );
+  connectFolder( folder );
+  return folder;
+}
+
 void KMailICalIfaceImpl::connectFolder( KMFolder* folder )
 {
   // avoid multiple connections
@@ -1895,8 +2193,8 @@ bool KMailICalIfaceImpl::folderIsAlarmRelevant( const KMFolder *folder )
     const KMFolderCachedImap *dimapFolder = static_cast<const KMFolderCachedImap*>( folder->storage() );
     administerRights =
       dimapFolder->userRights() <= 0 || dimapFolder->userRights() & KMail::ACLJobs::Administer;
-    relevantForOwner = dimapFolder->incidencesFor () == KMFolderCachedImap::IncForAdmins;
-    relevantForEveryone = ( dimapFolder->incidencesFor() == KMFolderCachedImap::IncForReaders );
+    relevantForOwner = !dimapFolder->alarmsBlocked() && ( dimapFolder->incidencesFor () == KMFolderCachedImap::IncForAdmins );
+    relevantForEveryone = !dimapFolder->alarmsBlocked() && ( dimapFolder->incidencesFor() == KMFolderCachedImap::IncForReaders );
   }
 #if 0
   kdDebug(5006) << k_funcinfo << endl;
@@ -1917,4 +2215,82 @@ bool KMailICalIfaceImpl::isResourceQuiet() const
   return mResourceQuiet;
 }
 
+
+bool KMailICalIfaceImpl::addSubresource( const QString& resource,
+                                         const QString& parent,
+                                         const QString& contentsType )
+{
+  kdDebug(5006) << "Adding subresource to parent: " << parent << " with name: " << resource << endl;
+  kdDebug(5006) << "contents type: " << contentsType << endl;
+  KMFolder *folder = findResourceFolder( parent );
+  KMFolderDir *parentFolderDir = !parent.isEmpty() && folder ? folder->createChildFolder(): mFolderParentDir;
+  if ( !parentFolderDir || parentFolderDir->hasNamedFolder( resource ) ) return false;
+
+  KMFolderType type = mFolderType;
+  if( type == KMFolderTypeUnknown ) type = KMFolderTypeMaildir;
+
+  KMFolder* newFolder = parentFolderDir->createFolder( resource, false, type );
+  if ( !newFolder ) return false;
+  if( mFolderType == KMFolderTypeImap )
+    static_cast<KMFolderImap*>( folder->storage() )->createFolder( resource );
+
+  StorageFormat defaultFormat = GlobalSettings::self()->theIMAPResourceStorageFormat() == GlobalSettings::EnumTheIMAPResourceStorageFormat::XML ? StorageXML : StorageIcalVcard;
+  setStorageFormat( newFolder, folder ? storageFormat( folder ) : defaultFormat );
+  newFolder->storage()->setContentsType( folderContentsType( contentsType ) );
+  newFolder->storage()->writeConfig();
+  newFolder->open( "ical_subresource" );
+  connectFolder( newFolder );
+  reloadFolderTree();
+
+  return true;
+}
+
+bool KMailICalIfaceImpl::removeSubresource( const QString& location )
+{
+  kdDebug(5006) << k_funcinfo << endl;
+
+  KMFolder *folder = findResourceFolder( location );
+
+  // We don't allow the default folders to be deleted, so check for
+  // those first. It would be nicer to produce a more meaningful error,
+  // or prevent deletion of the builtin folders from the gui already.
+  if ( !folder || isStandardResourceFolder( folder ) )
+      return false;
+
+  // the folder will be removed, which implies closed, so make sure
+  // nothing is using it anymore first
+  subresourceDeleted( folderContentsType( folder->storage()->contentsType() ), location );
+  mExtraFolders.remove( location );
+  folder->disconnect( this );
+
+  if ( folder->folderType() == KMFolderTypeImap )
+    kmkernel->imapFolderMgr()->remove( folder );
+  else if ( folder->folderType() == KMFolderTypeCachedImap ) {
+    // Deleted by user -> tell the account (see KMFolderCachedImap::listDirectory2)
+    KMFolderCachedImap* storage = static_cast<KMFolderCachedImap*>( folder->storage() );
+    KMAcctCachedImap* acct = storage->account();
+    if ( acct )
+      acct->addDeletedFolder( folder );
+    kmkernel->dimapFolderMgr()->remove( folder );
+  }
+  return true;
+}
+
+void KMailICalIfaceImpl::syncFolder(KMFolder * folder) const
+{
+  if ( kmkernel->isOffline() || !GlobalSettings::immediatlySyncDIMAPOnGroupwareChanges() )
+    return;
+  KMFolderCachedImap *dimapFolder = dynamic_cast<KMFolderCachedImap*>( folder->storage() );
+  if ( !dimapFolder )
+    return;
+  // check if the folder exists already, otherwise sync its parent as well to create it
+  if ( dimapFolder->imapPath().isEmpty() ) {
+    if ( folder->parent() && folder->parent()->owner() )
+      syncFolder( folder->parent()->owner() );
+    else
+      return;
+  }
+  dimapFolder->account()->processNewMailSingleFolder( folder );
+}
+
 #include "kmailicalifaceimpl.moc"
