@@ -51,12 +51,13 @@ class KMMessage;
 class KMFolderDir;
 class KMFolderTreeItem;
 
-namespace {
+namespace KMail {
 
   // Local helper class
 class ExtraFolder {
 public:
-  ExtraFolder( KMFolder* f ) : folder( f ) {}
+  ExtraFolder( KMFolder* f );
+  ~ExtraFolder();
   QGuardedPtr<KMFolder> folder;
 };
 
@@ -123,6 +124,18 @@ public:
                       Q_UINT32 sernum,
                       const QString& filename );
 
+  QString attachmentMimetype( const QString &resource,
+                              Q_UINT32 sernum,
+                              const QString &filename );
+
+  QStringList listAttachments( const QString &resource, Q_UINT32 sernum );
+
+
+  bool removeSubresource( const QString& );
+
+  bool addSubresource( const QString& resource,
+                       const QString& parent,
+                       const QString& contentsType );
 
   // tell KOrganizer about messages to be deleted
   void msgRemoved( KMFolder*, KMMessage* );
@@ -149,6 +162,13 @@ public:
    */
   bool hideResourceFolder( KMFolder* folder ) const;
 
+  /**
+   * Returns true if the given folder is the root of the groupware account,
+   * groupware folders are hidden, and only groupware folders shown in this
+   * account.
+   */
+  bool hideResourceAccountRoot( KMFolder* folder ) const;
+
   /**
    * Returns the resource folder type. Other is returned if resource
    * isn't enabled or it isn't a resource folder.
@@ -226,11 +246,14 @@ private slots:
 private:
   /** Helper function for initFolders. Initializes a single folder. */
   KMFolder* initFolder( KMail::FolderContentsType contentsType );
+  KMFolder* initScalixFolder( KMail::FolderContentsType contentsType );
 
   void connectFolder( KMFolder* folder );
 
   KMFolder* extraFolder( const QString& type, const QString& folder );
 
+  void syncFolder( KMFolder* folder ) const;
+
   struct StandardFolderSearchResult
   {
     enum FoundEnum { FoundAndStandard, NotFound, FoundByType, FoundByName };
@@ -267,6 +290,8 @@ private:
 
   StorageFormat globalStorageFormat() const;
 
+  static bool folderIsAlarmRelevant( const KMFolder * );
+
 private:
   QGuardedPtr<KMFolder> mContacts;
   QGuardedPtr<KMFolder> mCalendar;
@@ -276,9 +301,9 @@ private:
 
   // The extra IMAP resource folders
   // Key: folder location. Data: folder.
-  QDict<ExtraFolder> mExtraFolders;
+  QDict<KMail::ExtraFolder> mExtraFolders;
   // used for collecting incidences during async loading
-  QDict<Accumulator> mAccumulators;
+  QDict<KMail::Accumulator> mAccumulators;
   // More info for each folder we care about (mContacts etc. as well as the extra folders)
   // The reason for storing it here is that it can be shared between
   // kmfoldercachedimap and kmfolderimap, and that it's groupware data anyway.
