@@ -33,6 +33,7 @@
 #include "compactionjob.h"
 #include "kmfoldertree.h"
 #include "kmailicalifaceimpl.h"
+#include "kmaccount.h"
 
 #include <errno.h>
 
@@ -51,6 +52,7 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
     mIsSystemFolder( false ),
     mHasIndex( withIndex ),
     mExportsSernums( exportedSernums ),
+    mMoveInProgress( false ),
     mExpireMessages( false ), mUnreadExpireAge( 28 ),
     mReadExpireAge( 14 ), mUnreadExpireUnits( expireNever ),
     mReadExpireUnits( expireNever ),
@@ -84,12 +86,6 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
     }
   }
 
-   // trigger from here, since it needs a fully constructed FolderStorage
-  if ( mExportsSernums )
-    mStorage->registerWithMessageDict();
-  if ( !mHasIndex )
-    mStorage->setAutoCreateIndex( false );
-
   if ( aParent ) {
     connect( mStorage, SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ),
              aParent->manager(), SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ) );
@@ -131,10 +127,21 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
 
   connect( mStorage, SIGNAL( contentsTypeChanged( KMail::FolderContentsType ) ),
                 this, SLOT( slotContentsTypeChanged( KMail::FolderContentsType ) ) );
+  
+  connect( mStorage, SIGNAL( folderSizeChanged() ),
+           this, SLOT( slotFolderSizeChanged() ) );
 
   //FIXME: Centralize all the readConfig calls somehow - Zack
+  // Meanwhile, readConfig must be done before registerWithMessageDict, since
+  // that one can call writeConfig in some circumstances - David
   mStorage->readConfig();
 
+   // trigger from here, since it needs a fully constructed FolderStorage
+  if ( mExportsSernums )
+    mStorage->registerWithMessageDict();
+  if ( !mHasIndex )
+    mStorage->setAutoCreateIndex( false );
+
   if ( mId == 0 && aParent )
     mId = aParent->manager()->createId();
 }
@@ -203,7 +210,10 @@ void KMFolder::writeConfig( KConfig* config ) const
   config->writeEntry("MailingListEnabled", mMailingListEnabled);
   mMailingList.writeConfig( config );
 
-  config->writeEntry("Identity", mIdentity);
+  if ( mIdentity != 0 && ( !mStorage || !mStorage->account() || mIdentity != mStorage->account()->identityId() ) )
+      config->writeEntry("Identity", mIdentity);
+  else
+      config->deleteEntry("Identity");
 
   config->writeEntry("WhoField", mUserWhoField);
   config->writeEntry("Id", mId);
@@ -263,8 +273,13 @@ KMFolderDir* KMFolder::createChildFolder()
     }
   }
 
-  mChild = new KMFolderDir( this, parent(), childName,
-    (folderType() == KMFolderTypeImap) ? KMImapDir : KMStandardDir);
+  KMFolderDirType newType = KMStandardDir;
+  if( folderType() == KMFolderTypeCachedImap )
+    newType = KMDImapDir;
+  else if( folderType() == KMFolderTypeImap )
+    newType = KMImapDir;
+
+  mChild = new KMFolderDir( this, parent(), childName, newType );
   if( !mChild )
     return 0;
   mChild->reload();
@@ -280,7 +295,7 @@ void KMFolder::setChild( KMFolderDir* aChild )
 
 bool KMFolder::noContent() const
 {
-  return mStorage->noContent();
+  return mStorage ? mStorage->noContent() : true;
 }
 
 void KMFolder::setNoContent( bool aNoContent )
@@ -313,11 +328,6 @@ bool KMFolder::isMessage( int idx )
   return mStorage->isMessage( idx );
 }
 
-QCString& KMFolder::getMsgString( int idx, QCString& mDest )
-{
-  return mStorage->getMsgString( idx,  mDest );
-}
-
 DwString KMFolder::getDwString( int idx )
 {
   return mStorage->getDwString( idx );
@@ -367,7 +377,7 @@ KMMessage* KMFolder::take( int idx )
   return mStorage->take( idx );
 }
 
-void KMFolder::take( QPtrList<KMMessage> msgList )
+void KMFolder::take( QPtrList<KMMessage> msgList ) // TODO const ref
 {
   mStorage->take( msgList );
 }
@@ -397,7 +407,7 @@ void KMFolder::removeMsg( int i, bool imapQuiet )
   mStorage->removeMsg( i, imapQuiet );
 }
 
-void KMFolder::removeMsg( QPtrList<KMMessage> msgList, bool imapQuiet )
+void KMFolder::removeMsg( QPtrList<KMMessage> msgList, bool imapQuiet ) // TODO const ref
 {
   mStorage->removeMsg( msgList, imapQuiet );
 }
@@ -419,12 +429,12 @@ int KMFolder::moveMsg(QPtrList<KMMessage> q, int* index_return )
 
 int KMFolder::find( const KMMsgBase* msg ) const
 {
-  return mStorage->find( msg );
+  return mStorage ? mStorage->find( msg ) : -1;
 }
 
 int KMFolder::find( const KMMessage* msg ) const
 {
-  return mStorage->find( msg );
+  return mStorage ? mStorage->find( msg ) : -1;
 }
 
 int KMFolder::count( bool cache ) const
@@ -461,9 +471,9 @@ void KMFolder::msgStatusChanged( const KMMsgStatus oldStatus,
   mStorage->msgStatusChanged( oldStatus, newStatus, idx );
 }
 
-int KMFolder::open()
+int KMFolder::open(const char *owner)
 {
-  return mStorage->open();
+  return mStorage->open(owner);
 }
 
 int KMFolder::canAccess()
@@ -471,9 +481,10 @@ int KMFolder::canAccess()
   return mStorage->canAccess();
 }
 
-void KMFolder::close( bool force )
+void KMFolder::close( const char *owner, bool force )
 {
-  mStorage->close( force );
+  // do not emit closed() in here - as this would regain too early
+  mStorage->close( owner, force );
 }
 
 void KMFolder::sync()
@@ -498,6 +509,12 @@ void KMFolder::markUnreadAsRead()
 
 void KMFolder::remove()
 {
+  /* The storage needs to be open before remove is called, otherwise
+     it will not unregister the corresponding serial numbers from
+     the message dict, since its message list is empty, and the .ids
+     file contents are not loaded. That can lead to lookups in the
+     dict returning stale pointers to the folder later. */
+  mStorage->open("kmfolder_remove");
   mStorage->remove();
 }
 
@@ -598,6 +615,16 @@ void KMFolder::setIdentity( uint identity )
   kmkernel->slotRequestConfigSync();
 }
 
+uint KMFolder::identity() const
+{
+  // if we don't have one set ourselves, check our account
+  kdDebug() << "FOO: " << mIdentity << " :: " << mStorage << endl;
+  if ( !mIdentity && mStorage )
+    if ( KMAccount *act = mStorage->account() )
+      return act->identityId();
+  return mIdentity;
+}
+
 void KMFolder::setWhoField(const QString& aWhoField )
 {
   mWhoField = aWhoField;
@@ -624,11 +651,13 @@ void KMFolder::setUserWhoField( const QString& whoField, bool writeConfig )
         mWhoField = "From";
       if ( this == kmkernel->outboxFolder() ||
            this == kmkernel->sentFolder() ||
-           this == kmkernel->draftsFolder() )
+           this == kmkernel->draftsFolder() ||
+           this == kmkernel->templatesFolder() )
         mWhoField = "To";
-    } else if ( identity.drafts() == idString()
-                || identity.fcc() == idString() )
-      // drafts or sent of the identity
+    } else if ( identity.drafts() == idString() ||
+                identity.templates() == idString() ||
+                identity.fcc() == idString() )
+      // drafts, templates or sent of the identity
       mWhoField = "To";
     else
       mWhoField = "From";
@@ -693,6 +722,7 @@ void KMFolder::setUnreadExpireUnits( ExpireUnits units )
 {
   if (units >= expireNever && units < expireMaxUnits)
     mUnreadExpireUnits = units;
+    mStorage->writeConfig();
 }
 
 void KMFolder::setReadExpireAge( int age )
@@ -707,6 +737,7 @@ void KMFolder::setReadExpireUnits( ExpireUnits units )
 {
   if (units >= expireNever && units <= expireMaxUnits)
     mReadExpireUnits = units;
+    mStorage->writeConfig();
 }
 
 
@@ -827,7 +858,7 @@ void KMFolder::setShortcut( const KShortcut &sc )
 
 bool KMFolder::isMoveable() const
 {
-  return mStorage->isMoveable();
+  return !isSystemFolder();
 }
 
 void KMFolder::slotContentsTypeChanged( KMail::FolderContentsType type )
@@ -836,4 +867,14 @@ void KMFolder::slotContentsTypeChanged( KMail::FolderContentsType type )
   emit iconsChanged();
 }
 
+void KMFolder::slotFolderSizeChanged()
+{
+  emit folderSizeChanged( this );
+  KMFolder* papa = parent()->manager()->parentFolder( this );
+  if ( papa && papa != this ) {
+    papa->slotFolderSizeChanged();
+  }
+}
+
+
 #include "kmfolder.moc"
