@@ -1,4 +1,4 @@
-/**
+/*
  *  kmfoldercachedimap.cpp
  *
  *  Copyright (c) 2002-2004 Bo Thorsen <bo@sonofthor.dk>
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  In addition, as a special exception, the copyright holders give
  *  permission to link the code of this program with any edition of
@@ -41,13 +41,18 @@
 
 #include "kmfoldermaildir.h"
 #include "kmfolderimap.h"
-#include "kmacctimap.h"
+#include "kmacctcachedimap.h"
 #include "kmfoldertype.h"
 #include "folderjob.h"
 #include "cachedimapjob.h"
+#include "quotajobs.h"
 
 using KMail::FolderJob;
-class KMAcctCachedImap;
+using KMail::QuotaInfo;
+class KMCommand;
+
+class QComboBox;
+class QRadioButton;
 
 namespace KMail {
   class AttachmentStrategy;
@@ -60,15 +65,24 @@ class DImapTroubleShootDialog : public KDialogBase
 {
   Q_OBJECT
 public:
+  enum SelectedOperation {
+    None = -1,
+    ReindexCurrent = 0,
+    ReindexRecursive = 1,
+    ReindexAll = 2,
+    RefreshCache
+  };
+
   DImapTroubleShootDialog( QWidget* parent=0, const char* name=0 );
 
   static int run();
 
 private slots:
-  void slotRebuildIndex();
-  void slotRebuildCache();
+  void slotDone();
 
 private:
+  QRadioButton *mIndexButton, *mCacheButton;
+  QComboBox *mIndexScope;
   int rc;
 };
 
@@ -87,25 +101,38 @@ public:
   KMFolderCachedImap(KMFolder* folder, const char* name=0);
   virtual ~KMFolderCachedImap();
 
+  /**  @reimpl */
+  void reallyDoClose(const char* owner);
+
   /** Initialize this storage from another one. Used when creating a child folder */
   void initializeFrom( KMFolderCachedImap* parent );
 
   virtual void readConfig();
   virtual void writeConfig();
 
-  void writeAnnotationConfig();
+  void writeConfigKeysWhichShouldNotGetOverwrittenByReadConfig();
 
   /** Returns the type of this folder */
   virtual KMFolderType folderType() const { return KMFolderTypeCachedImap; }
 
+  /** @reimpl */
+  virtual int create();
+
   /** Remove this folder */
   virtual void remove();
 
   /** Synchronize this folder and it's subfolders with the server */
   virtual void serverSync( bool recurse );
 
-  /** Force the sync state to be done. */
-  void resetSyncState();
+  /**  Force the sync state to be done. */
+  void resetSyncState( );
+
+  /** Block this folder from generating alarms, even if the annotations
+   * on it say otherwise. Used to override alarms for read-only folders.
+   * (Only useful for resource folders) */
+  void setAlarmsBlocked( bool blocked );
+  /** Should alarms for this folder be blocked?  (Only useful for resource folders) */
+  bool alarmsBlocked() const;
 
   void checkUidValidity();
 
@@ -166,13 +193,14 @@ public:
   }
 
   /* Reimplemented from KMFolderMaildir */
-  virtual void removeMsg(int i, bool imapQuiet = FALSE);
-  virtual void removeMsg(QPtrList<KMMessage> msgList, bool imapQuiet = FALSE)
+  virtual void removeMsg(int i, bool imapQuiet = false);
+  virtual void removeMsg(QPtrList<KMMessage> msgList, bool imapQuiet = false)
     { FolderStorage::removeMsg(msgList, imapQuiet); }
 
   /// Is the folder readonly?
   bool isReadOnly() const { return KMFolderMaildir::isReadOnly() || mReadOnly; }
 
+
   /**
    * Emit the folderComplete signal
    */
@@ -188,6 +216,8 @@ public:
   virtual int createIndexFromContents()
     { return KMFolderMaildir::createIndexFromContents(); }
 
+  int createIndexFromContentsRecursive();
+
   //virtual void holdSyncs( bool hold ) { mHoldSyncs = hold; }
 
   /**
@@ -195,7 +225,9 @@ public:
    * It uses a ListJob to get the folders
    * returns false if the connection failed
    */
-  virtual bool listDirectory(bool secondStep = false);
+  virtual bool listDirectory();
+
+  virtual void listNamespaces();
 
   /** Return the trash folder. */
   KMFolder* trashFolder() const;
@@ -209,6 +241,18 @@ public:
   /// Set the user's rights on this folder - called by getUserRights
   void setUserRights( unsigned int userRights );
 
+  /**
+   * The quota information for this folder.
+   * @return an invalid info if we haven't synced yet, or the server
+   * doesn't support quota. The difference can be figured out by
+   * asking the account whether it supports quota. If we have
+   * synced, the account supports quota, but there is no quota
+   * on the folder, the return info will be valid, but empty.
+   * @see QuotaInfo::isEmpty(), QuotaInfo::isValid()
+   */
+  const QuotaInfo quotaInfo() const { return mQuotaInfo; }
+  void setQuotaInfo( const QuotaInfo & );
+
   /// Return the list of ACL for this folder
   typedef QValueVector<KMail::ACLListEntry> ACLList;
   const ACLList& aclList() const { return mACLList; }
@@ -217,8 +261,8 @@ public:
   void setACLList( const ACLList& arr );
 
   // Reimplemented so the mStatusChangedLocally bool can be set
-  virtual void setStatus( int id, KMMsgStatus status, bool toggle = false );
-  virtual void setStatus( QValueList<int>& ids, KMMsgStatus status, bool toggle = false );
+  virtual void setStatus( int id, KMMsgStatus status, bool toggle );
+  virtual void setStatus( QValueList<int>& ids, KMMsgStatus status, bool toggle );
 
   QString annotationFolderType() const { return mAnnotationFolderType; }
 
@@ -242,14 +286,30 @@ public:
   /** Returns true if this folder can be moved */
   virtual bool isMoveable() const;
 
-protected slots:
   /**
-   * Connected to ListJob::receivedFolders
-   * creates/removes folders
+   * List of namespaces that need to be queried
+   * Is set by the account for the root folder when the listing starts
    */
-  void slotListResult(const QStringList&, const QStringList&,
-      const QStringList&, const QStringList&, const ImapAccountBase::jobData& );
+  QStringList namespacesToList() { return mNamespacesToList; }
+  void setNamespacesToList( QStringList list ) { mNamespacesToList = list; }
 
+  /**
+   * Specify an imap path that is used to create the folder on the server
+   * Otherwise the parent folder is used to construct the path
+   */
+  const QString& imapPathForCreation() { return mImapPathCreation; }
+  void setImapPathForCreation( const QString& path ) { mImapPathCreation = path; }
+
+  /**  \reimp */
+  bool isCloseToQuota() const;
+
+  /** Flags that can be permanently stored on the server. */
+  int permanentFlags() const { return mPermanentFlags; }
+
+
+  QString folderAttributes() const { return mFolderAttributes; }
+
+protected slots:
   void slotGetMessagesData(KIO::Job * job, const QByteArray & data);
   void getMessagesResult(KMail::FolderJob *, bool lastSet);
   void slotGetLastMessagesResult(KMail::FolderJob *);
@@ -263,8 +323,10 @@ protected slots:
   void slotConnectionResult( int errorCode, const QString& errorMsg );
 
   void slotCheckUidValidityResult( KMail::FolderJob* job );
+  void slotPermanentFlags( int flags );
   void slotTestAnnotationResult(KIO::Job *job);
   void slotGetAnnotationResult( KIO::Job* );
+  void slotMultiUrlGetAnnotationResult( KIO::Job* );
   void slotSetAnnotationResult(KIO::Job *job);
   void slotReceivedUserRights( KMFolder* );
   void slotReceivedACL( KMFolder*, KIO::Job*, const KMail::ACLList& );
@@ -275,6 +337,8 @@ protected slots:
   void slotAnnotationChanged( const QString& entry, const QString& attribute, const QString& value );
   void slotDeleteMessagesResult(KMail::FolderJob *);
   void slotImapStatusChanged(KMFolder* folder, const QString&, bool);
+  void slotStorageQuotaResult( const QuotaInfo& );
+  void slotQuotaResult( KIO::Job* job );
 
 protected:
   /* returns true if there were messages to delete
@@ -283,9 +347,11 @@ protected:
   void listMessages();
   void uploadNewMessages();
   void uploadFlags();
+  void uploadSeenFlags();
   void createNewFolders();
 
   void listDirectory2();
+  void createFoldersNewOnServerAndFinishListing( const QValueVector<int> foldersNewOnServer );
 
 
   /** Utility methods for syncing. Finds new messages
@@ -309,6 +375,11 @@ protected:
   /* update progress status */
   void newState( int progress, const QString& syncStatus );
 
+  /** See if there is a better parent then this folder */
+  KMFolderCachedImap* findParent( const QString& path, const QString& name );
+
+
+
 public slots:
   /**
    * Add the data a KIO::Job retrieves to the buffer
@@ -320,10 +391,26 @@ public slots:
    */
   void slotTroubleshoot();
 
+  /**
+   * Connected to ListJob::receivedFolders
+   * creates/removes folders
+   */
+  void slotListResult( const QStringList&, const QStringList&,
+      const QStringList&, const QStringList&, const ImapAccountBase::jobData& );
+
+  /**
+   * Connected to ListJob::receivedFolders
+   * creates namespace folders
+   */
+  void slotCheckNamespace( const QStringList&, const QStringList&,
+      const QStringList&, const QStringList&, const ImapAccountBase::jobData& );
+
 private slots:
   void serverSyncInternal();
   void slotIncreaseProgress();
   void slotUpdateLastUid();
+  void slotFolderDeletionOnServerFinished();
+  void slotRescueDone( KMCommand* command );
 
 signals:
   void folderComplete(KMFolderCachedImap *folder, bool success);
@@ -337,6 +424,14 @@ signals:
 private:
   void setReadOnly( bool readOnly );
   QString state2String( int state ) const;
+  void rememberDeletion( int );
+  /** Rescue not yet synced messages to a lost+found folder in case
+    syncing is not possible because the folder has been deleted on the
+    server or write access to this folder has been revoked.
+  */
+  KMCommand* rescueUnsyncedMessages();
+  /** Recursive helper function calling the above method. */
+  void rescueUnsyncedMessagesAndDeleteFolder( KMFolder *folder, bool root = true );
 
   /** State variable for the synchronization mechanism */
   enum {
@@ -345,6 +440,7 @@ private:
     SYNC_STATE_PUT_MESSAGES,
     SYNC_STATE_UPLOAD_FLAGS,
     SYNC_STATE_CREATE_SUBFOLDERS,
+    SYNC_STATE_LIST_NAMESPACES,
     SYNC_STATE_LIST_SUBFOLDERS,
     SYNC_STATE_LIST_SUBFOLDERS2,
     SYNC_STATE_DELETE_SUBFOLDERS,
@@ -358,6 +454,7 @@ private:
     SYNC_STATE_SET_ANNOTATIONS,
     SYNC_STATE_GET_ACLS,
     SYNC_STATE_SET_ACLS,
+    SYNC_STATE_GET_QUOTA,
     SYNC_STATE_FIND_SUBFOLDERS,
     SYNC_STATE_SYNC_SUBFOLDERS,
     SYNC_STATE_CHECK_UIDVALIDITY,
@@ -372,6 +469,7 @@ private:
   imapState   mContentState, mSubfolderState;
   QStringList mSubfolderNames, mSubfolderPaths,
               mSubfolderMimeTypes, mSubfolderAttributes;
+  QString     mFolderAttributes;
   QString     mAnnotationFolderType;
   IncidencesFor mIncidencesFor;
 
@@ -414,14 +512,18 @@ private:
       mLastUid. See above for details. */
   ulong mTentativeHighestUid;
 
-  int mUserRights;
+  /** Used to determine whether listing messages yielded a sensible result.
+   * Only then is the deletion o messages (which relies on succesful
+   * listing) attempted, during the sync.  */
+  bool mFoundAnIMAPDigest;
+
+  int mUserRights, mOldUserRights;
   ACLList mACLList;
 
   bool mSilentUpload;
   bool mFolderRemoved;
   //bool mHoldSyncs;
   bool mRecurse;
-  bool mCreateInbox;
   /** Set to true by setStatus. Indicates that the client has changed
       the status of at least one mail. The mail flags will therefore be
       uploaded to the server, overwriting the server's notion of the status
@@ -431,6 +533,20 @@ private:
   bool mAnnotationFolderTypeChanged;
   /// Set to true when the "incidences-for" annotation needs to be set on the next sync
   bool mIncidencesForChanged;
+
+  QStringList mNamespacesToList;
+  int mNamespacesToCheck;
+  bool mPersonalNamespacesCheckDone;
+  QString mImapPathCreation;
+
+  QuotaInfo mQuotaInfo;
+  QMap<ulong,void*> mDeletedUIDsSinceLastSync;
+  bool mAlarmsBlocked;
+
+  QValueList<KMFolder*> mToBeDeletedAfterRescue;
+  int mRescueCommandCount;
+
+  int mPermanentFlags;
 };
 
 #endif /*kmfoldercachedimap_h*/
