@@ -28,8 +28,14 @@
 #include "kmfiltermgr.h"
 #include "kmfoldermgr.h"
 #include "folderstorage.h"
+#include "kmfolder.h"
+#include "kmfoldercachedimap.h"
+#include "kmacctcachedimap.h"
 #include "renamejob.h"
+#include "copyfolderjob.h"
+
 using KMail::RenameJob;
+using KMail::CopyFolderJob;
 
 //-----------------------------------------------------------------------------
 KMFolderMgr::KMFolderMgr(const QString& aBasePath, KMFolderDirType dirType):
@@ -177,6 +183,27 @@ KMFolder* KMFolderMgr::createFolder(const QString& fName, bool sysFldr,
 
   if (!aFolderDir)
     fldDir = &mDir;
+
+  // check if this is a dimap folder and the folder we want to create has been deleted
+  // since the last sync
+  if ( fldDir->owner() && fldDir->owner()->folderType() == KMFolderTypeCachedImap ) {
+    KMFolderCachedImap *storage = static_cast<KMFolderCachedImap*>( fldDir->owner()->storage() );
+    KMAcctCachedImap *account = storage->account();
+    // guess imap path
+    QString imapPath = storage->imapPath();
+    if ( !imapPath.endsWith( "/" ) )
+      imapPath += "/";
+    imapPath += fName;
+    if ( account->isDeletedFolder( imapPath ) || account->isDeletedFolder( imapPath + "/" )
+       || account->isPreviouslyDeletedFolder( imapPath )
+       || account->isPreviouslyDeletedFolder( imapPath + "/" ) ) {
+      KMessageBox::error( 0, i18n("A folder with the same name has been deleted since the last mail check."
+          "You need to check mails first before creating another folder with the same name."),
+          i18n("Could Not Create Folder") );
+      return 0;
+    }
+  }
+
   fld = fldDir->createFolder(fName, sysFldr, aFolderType);
   if (fld) {
     if ( fld->id() == 0 )
@@ -339,6 +366,22 @@ void KMFolderMgr::removeFolder(KMFolder* aFolder)
   aFolder->remove();
 }
 
+KMFolder* KMFolderMgr::parentFolder( KMFolder* folder )
+{
+  // find the parent folder by stripping "." and ".directory" from the name
+  KMFolderDir* fdir = folder->parent();
+  QString parentName = fdir->name();
+  parentName = parentName.mid( 1, parentName.length()-11 );
+  KMFolderNode* parent = fdir->hasNamedFolder( parentName );
+  if ( !parent && fdir->parent() ) // dimap obviously has a different structure
+    parent = fdir->parent()->hasNamedFolder( parentName );
+
+  KMFolder* parentF = 0;
+  if ( parent )
+    parentF = dynamic_cast<KMFolder*>( parent );
+  return parentF;
+}
+
 void KMFolderMgr::removeFolderAux(KMFolder* aFolder, bool success)
 {
   if (!success) {
@@ -354,16 +397,19 @@ void KMFolderMgr::removeFolderAux(KMFolder* aFolder, bool success)
       break;
     }
   }
+  KMFolder* parentF = parentFolder( aFolder );
+
+  // aFolder will be deleted by the next call!
   aFolder->parent()->remove(aFolder);
-  // find the parent folder by stripping "." and ".directory" from the name
-  QString parentName = fdir->name();
-  parentName = parentName.mid( 1, parentName.length()-11 );
-  KMFolderNode* parent = fdir->hasNamedFolder( parentName );
-  if ( !parent && fdir->parent() ) // dimap obviously has a different structure
-    parent = fdir->parent()->hasNamedFolder( parentName );
+
   // update the children state
-  if ( parent )
-    static_cast<KMFolder*>(parent)->storage()->updateChildrenState();
+  if ( parentF )
+  {
+    if ( parentF != aFolder )
+    {
+      parentF->storage()->updateChildrenState();
+    }
+  }
   else
     kdWarning(5006) << "Can not find parent folder" << endl;
 
@@ -534,6 +580,18 @@ void KMFolderMgr::renameFolder( KMFolder* folder, const QString& newName,
   RenameJob* job = new RenameJob( folder->storage(), newName, newParent );
   connect( job, SIGNAL( renameDone( QString, bool ) ),
       this, SLOT( slotRenameDone( QString, bool ) ) );
+  connect( job, SIGNAL( renameDone( QString, bool ) ),
+	   this, SIGNAL( folderMoveOrCopyOperationFinished() ) );
+  job->start();
+}
+
+//-----------------------------------------------------------------------------
+void KMFolderMgr::copyFolder( KMFolder* folder, KMFolderDir *newParent )
+{
+  kdDebug(5006) << "Copy folder: " << folder->prettyURL() << endl;
+  CopyFolderJob* job = new CopyFolderJob( folder->storage(), newParent );
+  connect( job, SIGNAL( folderCopyComplete( bool ) ),
+	   this, SIGNAL( folderMoveOrCopyOperationFinished() ) );
   job->start();
 }
 
