@@ -114,7 +114,7 @@ void TimeLabels::hideMousePos()
   mMousePos->hide();
 }
 
-void TimeLabels::setCellHeight(int height)
+void TimeLabels::setCellHeight(double height)
 {
   mCellHeight = height;
 }
@@ -251,6 +251,7 @@ void TimeLabels::setAgenda(KOAgenda* agenda)
   connect(mAgenda, SIGNAL(mousePosSignal(const QPoint &)), this, SLOT(mousePosChanged(const QPoint &)));
   connect(mAgenda, SIGNAL(enterAgenda()), this, SLOT(showMousePos()));
   connect(mAgenda, SIGNAL(leaveAgenda()), this, SLOT(hideMousePos()));
+  connect(mAgenda, SIGNAL(gridSpacingYChanged( double ) ), this, SLOT( setCellHeight( double ) ) );
 }
 
 
@@ -413,8 +414,9 @@ void KOAlternateLabel::setText( const QString &text ) {
 ////////////////////////////////////////////////////////////////////////////
 
 KOAgendaView::KOAgendaView(Calendar *cal,QWidget *parent,const char *name) :
-  KOEventView (cal,parent,name), mExpandButton( 0 ), mAllowAgendaUpdate( true ),
-  mUpdateItem( 0 )
+  KOrg::AgendaView (cal,parent,name), mExpandButton( 0 ), mAllowAgendaUpdate( true ),
+  mUpdateItem( 0 ),
+  mResource( 0 )
 {
   mSelectedDates.append(QDate::currentDate());
 
@@ -572,6 +574,8 @@ void KOAgendaView::connectAgenda( KOAgenda *agenda, QPopupMenu *popup,
 
   connect( agenda, SIGNAL( newStartSelectSignal() ),
            otherAgenda, SLOT( clearSelection() ) );
+  connect( agenda, SIGNAL( newStartSelectSignal() ),
+           SIGNAL( timeSpanSelectionChanged()) );
 
   connect( agenda, SIGNAL( editIncidenceSignal( Incidence * ) ),
                    SIGNAL( editIncidenceSignal( Incidence * ) ) );
@@ -774,15 +778,14 @@ void KOAgendaView::createDayLabels()
     dayLayout->addWidget(dayLabel);
 
     // if a holiday region is selected, show the holiday name
-    if ( KOGlobals::self()->holidays() ) {
-      QString text = KOGlobals::self()->holidays()->shortText( date );
-      if ( !text.isEmpty() ) {
-        // use a KOAlternateLabel so when the text doesn't fit any more a tooltip is used
-        KOAlternateLabel*label = new KOAlternateLabel( text, text, QString::null, mDayLabels );
-        label->setMinimumWidth(1);
-        label->setAlignment(AlignCenter);
-        dayLayout->addWidget(label);
-      }
+    QStringList texts = KOGlobals::self()->holiday( date );
+    QStringList::ConstIterator textit = texts.begin();
+    for ( ; textit != texts.end(); ++textit ) {
+      // use a KOAlternateLabel so when the text doesn't fit any more a tooltip is used
+      KOAlternateLabel*label = new KOAlternateLabel( (*textit), (*textit), QString::null, mDayLabels );
+      label->setMinimumWidth(1);
+      label->setAlignment(AlignCenter);
+      dayLayout->addWidget(label);
     }
 
 #ifndef KORG_NOPLUGINS
@@ -972,6 +975,8 @@ void KOAgendaView::updateEventDates( KOAgendaItem *item )
 
   Incidence *incidence = item->incidence();
   if ( !incidence ) return;
+  if ( !mChanger || !mChanger->beginChange(incidence) ) return;
+  Incidence *oldIncidence = incidence->clone();
 
   QTime startTime(0,0,0), endTime(0,0,0);
   if ( incidence->doesFloat() ) {
@@ -997,6 +1002,7 @@ void KOAgendaView::updateEventDates( KOAgendaItem *item )
     Event*ev = static_cast<Event*>(incidence);
     if( incidence->dtStart() == startDt && ev->dtEnd() == endDt ) {
       // No change
+      delete oldIncidence;
       return;
     }
     incidence->setDtStart( startDt );
@@ -1011,6 +1017,7 @@ void KOAgendaView::updateEventDates( KOAgendaItem *item )
 
     if( td->dtDue() == endDt ) {
       // No change
+      delete oldIncidence;
       return;
     }
   }
@@ -1175,6 +1182,10 @@ void KOAgendaView::updateEventDates( KOAgendaItem *item )
   KOIncidenceToolTip::remove( item );
   KOIncidenceToolTip::add( item, incidence, KOAgendaItem::toolTipGroup() );
 
+  mChanger->changeIncidence( oldIncidence, incidence );
+  mChanger->endChange(incidence);
+  delete oldIncidence;
+
   // don't update the agenda as the item already has the correct coordinates.
   // an update would delete the current item and recreate it, but we are still
   // using a pointer to that item! => CRASH
@@ -1228,6 +1239,9 @@ void KOAgendaView::showIncidences( const Incidence::List & )
 void KOAgendaView::insertIncidence( Incidence *incidence, const QDate &curDate,
                                     int curCol )
 {
+  if ( !filterByResource( incidence ) )
+    return;
+
   // FIXME: Use a visitor here, or some other method to get rid of the dynamic_cast's
   Event *event = dynamic_cast<Event *>( incidence );
   Todo  *todo  = dynamic_cast<Todo  *>( incidence );
@@ -1457,6 +1471,8 @@ void KOAgendaView::fillAgenda()
 
         if ( ! todo->hasDueDate() ) continue;  // todo shall not be displayed if it has no date
 
+        if ( !filterByResource( todo ) ) continue;
+
         // ToDo items shall be displayed for the day they are due, but only showed today if they are already overdue.
         // Already completed items can be displayed on their original due date
         bool overdue = todo->isOverdue();
@@ -1510,10 +1526,10 @@ void KOAgendaView::clearView()
   mAgenda->clear();
 }
 
-CalPrinter::PrintType KOAgendaView::printType()
+CalPrinterBase::PrintType KOAgendaView::printType()
 {
-  if ( currentDateCount() == 1 ) return CalPrinter::Day;
-  else return CalPrinter::Week;
+  if ( currentDateCount() == 1 ) return CalPrinterBase::Day;
+  else return CalPrinterBase::Week;
 }
 
 void KOAgendaView::updateEventIndicatorTop( int newY )
@@ -1562,7 +1578,7 @@ void KOAgendaView::slotTodoDropped( Todo *todo, const QPoint &gpos, bool allDay
       todo->setDtDue( newTime );
       todo->setFloats( allDay );
       todo->setHasDueDate( true );
-      if ( !mChanger->addIncidence( todo ) ) {
+      if ( !mChanger->addIncidence( todo, this ) ) {
         KODialogManager::errorSaveIncidence( this, todo );
       }
     }
@@ -1718,3 +1734,32 @@ void KOAgendaView::setIncidenceChanger( IncidenceChangerBase *changer )
   mAgenda->setIncidenceChanger( changer );
   mAllDayAgenda->setIncidenceChanger( changer );
 }
+
+void KOAgendaView::clearTimeSpanSelection()
+{
+  mAgenda->clearSelection();
+  mAllDayAgenda->clearSelection();
+  deleteSelectedDateTime();
+}
+
+void KOAgendaView::setResource(KCal::ResourceCalendar * res, const QString & subResource)
+{
+  mResource = res;
+  mSubResource = subResource;
+}
+
+bool KOAgendaView::filterByResource(Incidence * incidence)
+{
+  if ( !mResource )
+    return true;
+  CalendarResources *calRes = dynamic_cast<CalendarResources*>( calendar() );
+  if ( !calRes )
+    return true;
+  if ( calRes->resource( incidence ) != mResource )
+    return false;
+  if ( !mSubResource.isEmpty() ) {
+    if ( mResource->subresourceIdentifier( incidence ) != mSubResource )
+      return false;
+  }
+  return true;
+}
