@@ -36,7 +36,6 @@
 #include <kinstance.h>
 #include <kaboutdata.h>
 
-#include "uiDialog.h"
 #include "pluginfactory.h"
 
 #include "setup_base.h"
@@ -62,7 +61,7 @@ NullConduitConfig::NullConduitConfig(QWidget *p, const char *n) :
 {
 	FUNCTIONSETUP;
 	fConduitName = i18n("Null");
-		fAbout = new KAboutData("nullConduit",
+	fAbout = new KAboutData("nullConduit",
 		I18N_NOOP("Null Conduit for KPilot"),
 		KPILOT_VERSION,
 		I18N_NOOP("Configures the Null Conduit for KPilot"),
@@ -73,14 +72,10 @@ NullConduitConfig::NullConduitConfig(QWidget *p, const char *n) :
 		"groot@kde.org",
 		"http://www.cs.kun.nl/~adridg/kpilot");
 
-	UIDialog::addAboutPage(fConfigWidget->tabWidget,fAbout);
+	ConduitConfigBase::addAboutPage(fConfigWidget->tabWidget,fAbout);
 	fWidget=fConfigWidget;
 	QObject::connect(fConfigWidget->fLogMessage,SIGNAL(textChanged(const QString&)),
 		this,SLOT(modified()));
-	QObject::connect(fConfigWidget->fDatabases,SIGNAL(textChanged(const QString&)),
-		this,SLOT(modified()));
-	QObject::connect(fConfigWidget->fFailImmediately,SIGNAL(toggled(bool)),
-		this,SLOT(modified()));
 }
 
 /* virtual */ void NullConduitConfig::commit()
@@ -88,19 +83,13 @@ NullConduitConfig::NullConduitConfig(QWidget *p, const char *n) :
 	FUNCTIONSETUP;
 
 #ifdef DEBUG
-	DEBUGCONDUIT << fname
+	DEBUGKPILOT << fname
 		<< ": Message="
 		<< fConfigWidget->fLogMessage->text()
 		<< endl;
-	DEBUGCONDUIT << fname
-		<< ": Databases="
-		<< fConfigWidget->fDatabases->text()
-		<< endl;
 #endif
 
 	NullConduitSettings::setLogMessage( fConfigWidget->fLogMessage->text() );
-	NullConduitSettings::setDatabases( fConfigWidget->fDatabases->text() );
-	NullConduitSettings::setFailImmediately( fConfigWidget->fFailImmediately->isChecked());
 	NullConduitSettings::self()->writeConfig();
 	unmodified();
 }
@@ -111,18 +100,11 @@ NullConduitConfig::NullConduitConfig(QWidget *p, const char *n) :
 	NullConduitSettings::self()->readConfig();
 
 	fConfigWidget->fLogMessage->setText( NullConduitSettings::logMessage() );
-	fConfigWidget->fDatabases->setText( NullConduitSettings::databases().join(",") );
-	fConfigWidget->fFailImmediately->setChecked( NullConduitSettings::failImmediately() );
-
 #ifdef DEBUG
-	DEBUGCONDUIT << fname
+	DEBUGKPILOT << fname
 		<< ": Read Message="
 		<< fConfigWidget->fLogMessage->text()
 		<< endl;
-	DEBUGCONDUIT << fname
-		<< ": Read Database="
-		<< fConfigWidget->fDatabases->text()
-		<< endl;
 #endif
 
 	unmodified();
@@ -133,6 +115,7 @@ NullConduitConfig::NullConduitConfig(QWidget *p, const char *n) :
 extern "C"
 {
 
+unsigned long version_conduit_null = Pilot::PLUGIN_API;
 void *init_conduit_null()
 {
 	return new ConduitFactory<NullConduitConfig,NullConduit>(0,"nullconduit");
