@@ -22,17 +22,14 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-static const char *kpilotconfigdialog_id =
-	"$Id: kpilotConfigDialog.cc,v 1.46 2005/01/13 15:52:41 adridg Exp $";
-
 #include "options.h"
 
 #include <pi-version.h>
@@ -103,8 +100,6 @@ DeviceConfigPage::DeviceConfigPage(QWidget * w, const char *n ) : ConfigPage( w,
 #undef CM
 
 	fConduitName = i18n("Device");
-
-	(void) kpilotconfigdialog_id;
 }
 
 void DeviceConfigPage::load()
@@ -127,8 +122,7 @@ void DeviceConfigPage::load()
 		fConfigWidget->fWorkaround->setCurrentItem(1);
 		break;
 	default:
-		kdWarning() << k_funcinfo 
-			<< ": Unknown workaround number "
+		WARNINGKPILOT << "Unknown workaround number "
 			<< (int) KPilotSettings::workarounds()
 			<< endl;
 		KPilotSettings::setWorkarounds(KPilotSettings::eWorkaroundNone);
@@ -153,7 +147,7 @@ void DeviceConfigPage::load()
 			"probably unsupported and can cause problems. "
 			"Are you sure you want to use this device name?</qt>")
 			.arg(d),
-		i18n("Device Name too Long")
+		i18n("Device Name too Long"), i18n("Use"), i18n("Do Not Use")
 		) ;
 	}
 #endif
@@ -176,12 +170,11 @@ void DeviceConfigPage::load()
 	case 0 : KPilotSettings::setWorkarounds(KPilotSettings::eWorkaroundNone); break;
 	case 1 : KPilotSettings::setWorkarounds(KPilotSettings::eWorkaroundUSB); break;
 	default :
-		kdWarning() << k_funcinfo 
-			<< ": Unknown workaround number "
+		WARNINGKPILOT << "Unknown workaround number "
 			<< fConfigWidget->fWorkaround->currentItem()
 			<< endl;
 		KPilotSettings::setWorkarounds(KPilotSettings::eWorkaroundNone);
-			
+
 	}
 	KPilotConfig::updateConfigVersion();
 	KPilotSettings::self()->writeConfig();
@@ -202,8 +195,7 @@ void DeviceConfigPage::load()
 		fConfigWidget->fPilotSpeed->setEnabled(false);
 		break;
 	default:
-		kdWarning() << k_funcinfo
-			<< ": Unknown port type " << i << endl;
+		WARNINGKPILOT << "Unknown port type " << i << endl;
 	}
 }
 
@@ -224,7 +216,7 @@ void DeviceConfigPage::setEncoding()
 	QString enc = fConfigWidget->fPilotEncoding->currentText();
 	if (enc.isEmpty())
 	{
-		kdWarning() << k_funcinfo << "Empty encoding. Will ignore it"<<endl;
+		WARNINGKPILOT << "Empty encoding. Will ignore it." << endl;
 	}
 	else
 	{
@@ -242,7 +234,7 @@ SyncConfigPage::SyncConfigPage(QWidget * w, const char *n ) : ConfigPage( w, n )
 
 #define CM(a,b) connect(fConfigWidget->a,b,this,SLOT(modified()));
 	CM(fSpecialSync, SIGNAL(activated(int)));
-	CM(fFullBackupCheck, SIGNAL(toggled(bool)));
+	CM(fFullSyncCheck, SIGNAL(toggled(bool)));
 	CM(fScreenlockSecure, SIGNAL(toggled(bool)));
 	CM(fConflictResolution, SIGNAL(activated(int)));
 #undef CM
@@ -250,13 +242,12 @@ SyncConfigPage::SyncConfigPage(QWidget * w, const char *n ) : ConfigPage( w, n )
 	fConduitName = i18n("HotSync");
 }
 
-#define MENU_ITEM_COUNT (5)
-static SyncAction::SyncMode syncTypeMap[MENU_ITEM_COUNT] = {
-	SyncAction::eHotSync,
-	SyncAction::eFastSync,
-	SyncAction::eFullSync,
-	SyncAction::eCopyPCToHH,
-	SyncAction::eCopyHHToPC
+#define MENU_ITEM_COUNT (4)
+static SyncAction::SyncMode::Mode syncTypeMap[MENU_ITEM_COUNT] = {
+	SyncAction::SyncMode::eHotSync,
+	SyncAction::SyncMode::eFullSync,
+	SyncAction::SyncMode::eCopyPCToHH,
+	SyncAction::SyncMode::eCopyHHToPC
 	} ;
 
 void SyncConfigPage::load()
@@ -266,7 +257,7 @@ void SyncConfigPage::load()
 
 	/* Sync tab */
 	int synctype=KPilotSettings::syncType();
-	if (synctype<0) synctype=(int) SyncAction::eHotSync;
+	if (synctype<0) synctype=(int) SyncAction::SyncMode::eHotSync;
 	for (unsigned int i=0; i<MENU_ITEM_COUNT; ++i)
 	{
 		if (syncTypeMap[i] == synctype)
@@ -281,7 +272,7 @@ void SyncConfigPage::load()
 		fConfigWidget->fSpecialSync->setCurrentItem(0); /* HotSync */
 	}
 
-	fConfigWidget->fFullBackupCheck->setChecked(KPilotSettings::fullSyncOnPCChange());
+	fConfigWidget->fFullSyncCheck->setChecked(KPilotSettings::fullSyncOnPCChange());
 	fConfigWidget->fConflictResolution->setCurrentItem(KPilotSettings::conflictResolution());
 	fConfigWidget->fScreenlockSecure->setChecked(KPilotSettings::screenlockSecure());
 
@@ -301,11 +292,11 @@ void SyncConfigPage::load()
 	}
 	if (synctype < 0)
 	{
-		synctype = SyncAction::eHotSync;
+		synctype = SyncAction::SyncMode::eHotSync;
 	}
 
 	KPilotSettings::setSyncType(synctype);
-	KPilotSettings::setFullSyncOnPCChange(fConfigWidget->fFullBackupCheck->isChecked());
+	KPilotSettings::setFullSyncOnPCChange(fConfigWidget->fFullSyncCheck->isChecked());
 	KPilotSettings::setConflictResolution(fConfigWidget->fConflictResolution->currentItem());
 	KPilotSettings::setScreenlockSecure(fConfigWidget->fScreenlockSecure->isChecked());
 
@@ -315,7 +306,6 @@ void SyncConfigPage::load()
 }
 
 
-
 BackupConfigPage::BackupConfigPage(QWidget * w, const char *n ) : ConfigPage( w, n )
 {
 	FUNCTIONSETUP;
@@ -332,6 +322,7 @@ BackupConfigPage::BackupConfigPage(QWidget * w, const char *n ) : ConfigPage( w,
 #define CM(a,b) connect(fConfigWidget->a,b,this,SLOT(modified()));
 	CM(fBackupOnly, SIGNAL(textChanged(const QString &)));
 	CM(fSkipDB, SIGNAL(textChanged(const QString &)));
+	CM(fBackupFrequency, SIGNAL(activated(int)));
 #undef CM
 
 	fConduitName = i18n("Backup");
@@ -346,6 +337,11 @@ void BackupConfigPage::load()
 	fConfigWidget->fBackupOnly->setText(KPilotSettings::skipBackupDB().join(CSL1(",")));
 	fConfigWidget->fSkipDB->setText(KPilotSettings::skipRestoreDB().join(CSL1(",")));
 	fConfigWidget->fRunConduitsWithBackup->setChecked(KPilotSettings::runConduitsWithBackup());
+
+	int backupfreq=KPilotSettings::backupFrequency();
+
+	fConfigWidget->fBackupFrequency->setCurrentItem(backupfreq);
+
 	unmodified();
 }
 
@@ -359,6 +355,7 @@ void BackupConfigPage::load()
 	KPilotSettings::setSkipRestoreDB(
 		QStringList::split(CSL1(","),fConfigWidget->fSkipDB->text()));
 	KPilotSettings::setRunConduitsWithBackup(fConfigWidget->fRunConduitsWithBackup->isChecked());
+	KPilotSettings::setBackupFrequency(fConfigWidget->fBackupFrequency->currentItem());
 
 	KPilotConfig::updateConfigVersion();
 	KPilotSettings::self()->writeConfig();
@@ -500,11 +497,11 @@ void StartExitConfigPage::load()
 	}
 
 #ifdef DEBUG
-	DEBUGDAEMON << fname << ": Autostart=" << autostart << endl;
-	DEBUGDAEMON << fname << ": desktop=" << desktopfile << endl;
-	DEBUGDAEMON << fname << ": location=" << location << endl;
+	DEBUGKPILOT << fname << ": Autostart=" << autostart << endl;
+	DEBUGKPILOT << fname << ": desktop=" << desktopfile << endl;
+	DEBUGKPILOT << fname << ": location=" << location << endl;
 #endif
-	
+
 	KPilotSettings::setStartDaemonAtLogin(fConfigWidget->fStartDaemonAtLogin->isChecked());
 	if (KPilotSettings::startDaemonAtLogin())
 	{
