@@ -1,37 +1,35 @@
 /* KPilot
-**
-** Copyright (C) 1998-2001 by Dan Pilone
-** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
-** Copyright (C) 2006-2007 Adriaan de Groot <groot@kde.org>
-** Copyright (C) 2007 Jason 'vanRijn' Kasper <vR@movingparts.net>
-**
-*/
+ **
+ ** Copyright (C) 1998-2001 by Dan Pilone
+ ** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
+ ** Copyright (C) 2006-2007 Adriaan de Groot <groot@kde.org>
+ ** Copyright (C) 2007 Jason 'vanRijn' Kasper <vR@movingparts.net>
+ **
+ */
 
 /*
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU Lesser General Public License as published by
-** the Free Software Foundation; either version 2.1 of the License, or
-** (at your option) any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU Lesser General Public License for more details.
-**
-** You should have received a copy of the GNU Lesser General Public License
-** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-** MA 02110-1301, USA.
-*/
+ ** This program is free software; you can redistribute it and/or modify
+ ** it under the terms of the GNU Lesser General Public License as published by
+ ** the Free Software Foundation; either version 2.1 of the License, or
+ ** (at your option) any later version.
+ **
+ ** This program is distributed in the hope that it will be useful,
+ ** but WITHOUT ANY WARRANTY; without even the implied warranty of
+ ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ ** GNU Lesser General Public License for more details.
+ **
+ ** You should have received a copy of the GNU Lesser General Public License
+ ** along with this program in a file called COPYING; if not, write to
+ ** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ ** MA 02110-1301, USA.
+ */
 
 /*
-** Bug reports and questions can be sent to kde-pim@kde.org
-*/
+ ** Bug reports and questions can be sent to kde-pim@kde.org
+ */
 
 #include "options.h"
 
-
-
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <stdio.h>
@@ -51,6 +49,7 @@
 #include <qtimer.h>
 #include <qdatetime.h>
 #include <qthread.h>
+#include <qsocketnotifier.h>
 
 #include <kconfig.h>
 #include <kmessagebox.h>
@@ -65,254 +64,110 @@
 #include "pilotLocalDatabase.h"
 
 #include "kpilotlink.h"
+#include "kpilotdevicelinkPrivate.moc"
 #include "kpilotdevicelink.moc"
 
 
-// singleton helper class
-class DeviceMap
-{
-public:
-	static DeviceMap *self()
-	{
-		if (!mThis) mThis = new DeviceMap();
-		return mThis;
-	}
-
-	bool canBind( const QString &device )
-	{
-		showList();
-		return !mBoundDevices.contains( device );
-	}
-
-	void bindDevice( const QString &device )
-	{
-		mBoundDevices.append( device );
-		showList();
-	}
-
-	void unbindDevice( const QString &device )
-	{
-		mBoundDevices.remove( device );
-		showList();
-	}
-
-protected:
-	DeviceMap() {}
-	~DeviceMap() {}
-
-	QStringList mBoundDevices;
-	static DeviceMap *mThis;
-
-private:
-	inline void showList() const
-	{
-		if ( !(mBoundDevices.count() > 0) ) return;
-		FUNCTIONSETUPL(3);
-		DEBUGKPILOT << fname << ": Bound devices: ["
-			<< ((mBoundDevices.count() > 0) ?
-			mBoundDevices.join(CSL1(", ")) : CSL1("<none>")) 
-			<< "]" << endl;
-	}
-} ;
-
 DeviceMap *DeviceMap::mThis = 0L;
 
-class Messages
+
+static inline void startOpenTimer(DeviceCommThread *dev, QTimer *&t)
 {
-public:
-	Messages(KPilotDeviceLink *parent) :
-		fDeviceLink(parent)
+	if ( !t)
 	{
-		reset();
+		t = new QTimer(dev);
+		QObject::connect(t, SIGNAL(timeout()), dev, SLOT(openDevice()));
 	}
+	// just a single-shot timer.  we'll know when to start it again...
+	t->start(1000, true);
+}
 
-	void reset()
-	{
-		messages = 0;
-		messagesMask = ~messageIsError; // Never block errors
-	}
-
-	void block(unsigned int m, bool force=false)
-	{
-		if (force)
-		{
-			// Force blocking this message, even if it's an error msg.
-			messages |= m;
-		}
-		else
-		{
-			messages |= (m & messagesMask);
-		}
-	}
-
-	/**
-	* Some messages are only printed once and are suppressed
-	* after that. These are indicated by flag bits in
-	* messages. The following enum is a bitfield.
-	*/
-	enum {
-		OpenMessage=1, ///< Trying to open device ..
-		OpenFailMessage=2 ///< Failed to open device ...
-	} ;
-	int messages;
-	int messagesMask;
-	static const int messageIsError = 0;
-
-	/** Determines whether message @p s which has an id of @p msgid (one of
-	 *  the enum values mentioned above) should be printed, which is only if that
-	 *  message has not been suppressed through messagesMask.
-	 *  If return is true, this method also adds it to the messagesMask.
-	 */
-	bool shouldPrint(int msgid)
-	{
-		if (!(messages & msgid))
-		{
-			block(msgid);
-			return true;
-		}
-		else
-		{
-			return false;
-		}
-	}
-	
-protected:
-	KPilotDeviceLink *fDeviceLink;
-} ;
-
-class DeviceCommEvent : public QCustomEvent
+DeviceCommThread::DeviceCommThread(KPilotDeviceLink *d) :
+	QThread(),
+	fDone(true),
+	fHandle(d),
+	fOpenTimer(0L),
+	fSocketNotifier(0L),
+	fSocketNotifierActive(false),
+	fWorkaroundUSBTimer(0L),
+	fPilotSocket(-1),
+	fTempSocket(-1),
+	fAcceptedCount(0)
 {
-public:
-	DeviceCommEvent ( DeviceCustomEvents type, QString msg = QString::null, int progress = 0)
-		: QCustomEvent( type ),
-		fMessage( msg ),
-		fProgress( progress ),
-		fPilotSocket(-1) {}
-	QString message() const
-	{
-		return fMessage;
-	}
-	int progress()
-	{
-		return fProgress;
-	}
+	FUNCTIONSETUP;
+}
 
-	inline void setCurrentSocket(int i) { fPilotSocket = i; }
-
-	inline int currentSocket() { return fPilotSocket; }
-private:
-	QString fMessage;
-	int fProgress;
-	/**
-	* Pilot-link library handles for the device once it's opened.
-	*/
-	int fPilotSocket;
-};
-
-
-/** Class that handles all device communications.  We do this
-    in a different thread so that we do not block the main Qt
-    Event thread (similar to Swing's AWT event dispatch thread).
-*/
-
-class DeviceCommThread : public QThread
-{
-friend class KPilotDeviceLink;
-public:
-	DeviceCommThread(KPilotDeviceLink *d) :
-		QThread(),
-		fDone(true),
-		fHandle(d),
-		fPilotSocket(-1),
-		fTempSocket(-1)
-	{ };
-	virtual ~DeviceCommThread();
-
-	virtual void run();
-
-	static const int SecondsBetweenPoll = 1;
-
-	void setDone(bool b) 
-	{ 
-		FUNCTIONSETUP;
-		fDone = b; 
-	}
-
-private:
-	volatile bool fDone;
-
-	KPilotDeviceLink *fHandle;
-	inline KPilotDeviceLink *link() 
-	{
-		if (fHandle) 
-		{
-			return fHandle;
-		}
-		else
-		{
-			FUNCTIONSETUP;
-			WARNINGKPILOT << "Link asked for, but either I'm "
-				<< "done or I don't have a valid handle.  "
-				<< "Shutting down comm thread." << endl;
-			QThread::exit();
-			return 0;
-		}
-	}
-
-	/**
-	* Pilot-link library handles for the device once it's opened.
-	*/
-	int fPilotSocket;
-	int fTempSocket;
-
-protected:
-	/**
-	* Attempt to open the device. Called regularly to check
-	* if the device exists (to handle USB-style devices).  This does
-	* everything necessary to connect to a Palm device.  If the
-	* open was successful, this method returns true, otherwise false.
-	*/
-	bool openDevice();
-
-	void close();
-
-protected:
-	/**
-	* Does the low-level opening of the device and handles the
-	* pilot-link library initialisation.
-	*/
-	bool open( const QString &device = QString::null );
-	
-private:
-	inline QString errorMessage(int e) 
-	{
-		switch (e)
-		{
-		case ENOENT:
-			return i18n(" The port does not exist.");
-			break;
-		case ENODEV:
-			return i18n(" There is no such device.");
-			break;
-		case EPERM:
-			return i18n(" You do not have permission to open the "
-				"Pilot device.");
-			break;
-		default:
-			return i18n(" Check Pilot path and permissions.");
-		}
-	}
-
-} ;
 
 DeviceCommThread::~DeviceCommThread()
 {
 	FUNCTIONSETUPL(2);
 	close();
+	KPILOT_DELETE(fWorkaroundUSBTimer);
 }
 
+void DeviceCommThread::close()
+{
+	FUNCTIONSETUPL(2);
 
-bool DeviceCommThread::openDevice()
+	KPILOT_DELETE(fWorkaroundUSBTimer);
+	KPILOT_DELETE(fOpenTimer);
+	KPILOT_DELETE(fSocketNotifier);
+	fSocketNotifierActive=false;
+
+	if (fTempSocket != -1)
+	{
+		DEBUGKPILOT << fname
+		<< ": device comm thread closing socket: ["
+		<< fTempSocket << "]" << endl;
+
+		pi_close(fTempSocket);
+	}
+
+	if (fPilotSocket != -1)
+	{
+		DEBUGKPILOT << fname
+		<< ": device comm thread closing socket: ["
+		<< fPilotSocket << "]" << endl;
+
+		pi_close(fPilotSocket);
+	}
+
+	fTempSocket = (-1);
+	fPilotSocket = (-1);
+
+	DeviceMap::self()->unbindDevice(link()->fRealPilotPath);
+}
+
+void DeviceCommThread::reset()
+{
+	FUNCTIONSETUP;
+
+	if (link()->fMessages->shouldPrint(Messages::OpenFailMessage))
+	{
+		QApplication::postEvent(link(), new DeviceCommEvent(EventLogMessage,
+				i18n("Could not open device: %1 (will retry)")
+				.arg(link()->pilotPath() )));
+	}
+
+	link()->fMessages->reset();
+	close();
+
+	// Timer already deleted by close() call.
+	startOpenTimer(this,fOpenTimer);
+
+	link()->fLinkStatus = WaitingForDevice;
+}
+
+/**
+ * This is an asyncronous process.  We try to create a socket with the Palm
+ * and then bind to it (in open()).  If we're able to do those 2 things, then
+ * we do 2 things:  we set a timeout timer (which will tell us that X amount of
+ * time has transpired before we get into the meat of the sync transaction), and
+ * we also set up a QSocketNotifier, which will tell us when data is available
+ * to be read from the Palm socket.  If we were unable to create a socket
+ * and/or bind to the Palm in this method, we'll start our timer again.
+ */
+void DeviceCommThread::openDevice()
 {
 	FUNCTIONSETUPL(2);
 
@@ -329,37 +184,39 @@ bool DeviceCommThread::openDevice()
 	if (link()->fMessages->shouldPrint(Messages::OpenMessage))
 	{
 		QApplication::postEvent(link(), new DeviceCommEvent(EventLogMessage,
-			i18n("Trying to open device %1...")
-			.arg(link()->fPilotPath)));
+				i18n("Trying to open device %1...")
+				.arg(link()->fPilotPath)));
 	}
 
 	// if we're not supposed to be done, try to open the main pilot
 	// path...
 	if (!fDone && link()->fPilotPath.length() > 0)
 	{
-		DEBUGKPILOT << fname << ": Opening main pilot path: [" 
+		DEBUGKPILOT << fname << ": Opening main pilot path: ["
 			<< link()->fPilotPath << "]." << endl;
-		deviceOpened = open( link()->fPilotPath ); 
+		deviceOpened = open(link()->fPilotPath);
 	}
 
 	// only try the temp device if our earlier attempt didn't work and the temp
 	// device is different than the main device, and it's a non-empty
 	// string
-	bool tryTemp = !deviceOpened && 
-			(link()->fTempDevice.length() > 0) &&
-			(link()->fPilotPath != link()->fTempDevice) ;
+	bool tryTemp = !deviceOpened && (link()->fTempDevice.length() > 0) && (link()->fPilotPath != link()->fTempDevice);
 
 	// if we're not supposed to be done, and we should try the temp
 	// device, try the temp device...
 	if (!fDone && tryTemp)
 	{
 		DEBUGKPILOT << fname << ": Couldn't open main pilot path. "
-			<< "Now trying temp device: [" 
+			<< "Now trying temp device: ["
 			<< link()->fTempDevice << "]." << endl;
-		deviceOpened = open( link()->fTempDevice );
+		deviceOpened = open(link()->fTempDevice);
 	}
 
-	return deviceOpened;
+	// if we couldn't connect, try to connect again...
+	if (!fDone && !deviceOpened)
+	{
+		startOpenTimer(this, fOpenTimer);
+	}
 }
 
 bool DeviceCommThread::open(const QString &device)
@@ -370,24 +227,30 @@ bool DeviceCommThread::open(const QString &device)
 	int e = 0;
 	QString msg;
 
-	link()->fRealPilotPath = KStandardDirs::realFilePath(device.isEmpty() ? link()->fPilotPath : device );
+	if (fTempSocket != -1)
+	{
+		pi_close(fTempSocket);
+	}
+	fTempSocket = (-1);
 
-	if ( !DeviceMap::self()->canBind( link()->fRealPilotPath ) ) {
+	link()->fRealPilotPath
+			= KStandardDirs::realFilePath(device.isEmpty() ? link()->fPilotPath : device);
+
+	if ( !DeviceMap::self()->canBind(link()->fRealPilotPath) )
+	{
 		msg = i18n("Already listening on that device");
-		
+
 		WARNINGKPILOT << "Pilot Path: ["
 			<< link()->fRealPilotPath << "] already connected." << endl;
 		WARNINGKPILOT << msg << endl;
 
 		link()->fLinkStatus = PilotLinkError;
 
-		QApplication::postEvent(link(),
-			new DeviceCommEvent(EventLogError, msg));
+		QApplication::postEvent(link(), new DeviceCommEvent(EventLogError, msg));
 
 		return false;
 	}
 
-
 	DEBUGKPILOT << fname << ": Trying to create socket." << endl;
 
 	fTempSocket = pi_socket(PI_AF_PILOT, PI_SOCK_STREAM, PI_PF_DLP);
@@ -396,15 +259,14 @@ bool DeviceCommThread::open(const QString &device)
 	{
 		e = errno;
 		msg = i18n("Cannot create socket for communicating "
-			"with the Pilot (%1)").arg(errorMessage(e));
-		WARNINGKPILOT << msg << endl;
-		WARNINGKPILOT << "(" << strerror(e) << ")" << endl;
-	
+				"with the Pilot (%1)").arg(errorMessage(e));
+		DEBUGKPILOT << msg << endl;
+		DEBUGKPILOT << "(" << strerror(e) << ")" << endl;
+
 		link()->fLinkStatus = PilotLinkError;
-		
-		QApplication::postEvent(link(),
-			new DeviceCommEvent(EventLogError, msg));
-	
+
+		QApplication::postEvent(link(), new DeviceCommEvent(EventLogError, msg));
+
 		return false;
 	}
 
@@ -412,7 +274,7 @@ bool DeviceCommThread::open(const QString &device)
 
 	link()->fLinkStatus = CreatedSocket;
 
-	DEBUGKPILOT << fname << ": Binding to path: [" 
+	DEBUGKPILOT << fname << ": Binding to path: ["
 		<< link()->fRealPilotPath << "]" << endl;
 
 	ret = pi_bind(fTempSocket, QFile::encodeName(link()->fRealPilotPath));
@@ -425,23 +287,91 @@ bool DeviceCommThread::open(const QString &device)
 
 		e = errno;
 		msg = i18n("Cannot open Pilot port \"%1\". ").arg(link()->fRealPilotPath);
-		
-		WARNINGKPILOT << msg << endl;
-		WARNINGKPILOT << "(" << strerror(e) << ")" << endl;
-	
+
+		DEBUGKPILOT << msg << endl;
+		DEBUGKPILOT << "(" << strerror(e) << ")" << endl;
+
 		link()->fLinkStatus = PilotLinkError;
-		
+
 		if (link()->fMessages->shouldPrint(Messages::OpenFailMessage))
 		{
-			QApplication::postEvent(link(),
-				new DeviceCommEvent(EventLogError, msg));
+			QApplication::postEvent(link(), new DeviceCommEvent(EventLogError, msg));
 		}
 
 		return false;
 	}
 
 	link()->fLinkStatus = DeviceOpen;
-	DeviceMap::self()->bindDevice( link()->fRealPilotPath );
+	DeviceMap::self()->bindDevice(link()->fRealPilotPath);
+
+	fSocketNotifier = new QSocketNotifier(fTempSocket,
+			QSocketNotifier::Read, this);
+	QObject::connect(fSocketNotifier, SIGNAL(activated(int)),
+			this, SLOT(acceptDevice()));
+	fSocketNotifierActive=true;
+
+	/**
+	 * We _always_ want to set a maximum amount of time that we will wait
+	 * for the sync process to start.  In the case where our user
+	 * has told us that he has a funky USB device, set the workaround timeout
+	 * for shorter than normal.
+	 */
+	int timeout=20000;
+	if (link()->fWorkaroundUSB)
+	{
+		timeout=5000;
+	}
+
+	fWorkaroundUSBTimer = new QTimer(this);
+	connect(fWorkaroundUSBTimer, SIGNAL(timeout()), this, SLOT(workaroundUSB()));
+	fWorkaroundUSBTimer->start(timeout, true);
+
+	return true;
+}
+
+/**
+ * We've been notified by our QSocketNotifier that we have data available on the
+ * socket.  Try to go through the remaining steps of the connnection process.
+ * Note: If we return at all from this before the very end without a successful
+ * connection, we need to make sure we restart our connection open timer, otherwise
+ * it won't be restarted.
+ */
+void DeviceCommThread::acceptDevice()
+{
+	FUNCTIONSETUP;
+
+	int ret;
+
+	/**
+	 * Our socket notifier should be the only reason that we end up here.
+	 * If we're here without him being active, we have a problem.  Try to clean
+	 * up and get out.
+	 */
+	if (!fSocketNotifierActive)
+	{
+		if (!fAcceptedCount)
+		{
+		kdWarning() << k_funcinfo << ": Accidentally in acceptDevice()"
+			<< endl;
+		}
+		fAcceptedCount++;
+		if (fAcceptedCount>10)
+		{
+			// Damn the torpedoes
+			KPILOT_DELETE(fSocketNotifier);
+		}
+		return;
+	}
+
+	if (fSocketNotifier)
+	{
+		// fSocketNotifier->setEnabled(false);
+		fSocketNotifierActive=false;
+		KPILOT_DELETE(fSocketNotifier);
+	}
+
+	DEBUGKPILOT << fname << ": Found connection on device: ["
+		<< link()->pilotPath().latin1() << "]." <<endl;
 
 	DEBUGKPILOT << fname
 		<< ": Current status: ["
@@ -457,25 +387,19 @@ bool DeviceCommThread::open(const QString &device)
 
 		// Presumably, strerror() returns things in
 		// local8Bit and not latin1.
-		QApplication::postEvent(link(),
-			new DeviceCommEvent(EventLogError,
+		QApplication::postEvent(link(), new DeviceCommEvent(EventLogError,
 				i18n("Cannot listen on Pilot socket (%1)").
-				arg(QString::fromLocal8Bit(s)))
-			);
-
-		return false;
+				arg(QString::fromLocal8Bit(s))));
+		reset();
+		return;
 	}
 
-	QApplication::postEvent(link(),
-		new DeviceCommEvent(EventLogProgress, QString::null, 10));
+	QApplication::postEvent(link(), new DeviceCommEvent(EventLogProgress, QString::null, 10));
 
 	DEBUGKPILOT << fname <<
 		": Listening to pilot. Now trying accept..." << endl;
 
-	int timeout=20;
-	if (link()->fWorkaroundUSB)
-		timeout=10;
-
+	int timeout = 20;
 	fPilotSocket = pi_accept_to(fTempSocket, 0, 0, timeout);
 
 	if (fPilotSocket < 0)
@@ -484,12 +408,12 @@ bool DeviceCommThread::open(const QString &device)
 
 		WARNINGKPILOT << "pi_accept returned: [" << s << "]" << endl;
 
-		QApplication::postEvent(link(),
-			new DeviceCommEvent(EventLogError, i18n("Cannot accept Pilot (%1)")
-			.arg(QString::fromLocal8Bit(s))));
+		QApplication::postEvent(link(), new DeviceCommEvent(EventLogError, i18n("Cannot accept Pilot (%1)")
+				.arg(QString::fromLocal8Bit(s))));
 
 		link()->fLinkStatus = PilotLinkError;
-		return false;
+		reset();
+		return;
 	}
 
 	DEBUGKPILOT << fname << ": Link accept done." << endl;
@@ -499,33 +423,32 @@ bool DeviceCommThread::open(const QString &device)
 		link()->fLinkStatus = PilotLinkError;
 		WARNINGKPILOT << "Already connected or unable to connect!" << endl;
 
-		QApplication::postEvent(link(),
-			new DeviceCommEvent(EventLogError, i18n("Cannot accept Pilot (%1)")
-			.arg(i18n("already connected"))));
+		QApplication::postEvent(link(), new DeviceCommEvent(EventLogError, i18n("Cannot accept Pilot (%1)")
+				.arg(i18n("already connected"))));
 
-		return false;
+		reset();
+		return;
 	}
 
-	QApplication::postEvent(link(),
-		new DeviceCommEvent(EventLogProgress, QString::null, 30));
+	QApplication::postEvent(link(), new DeviceCommEvent(EventLogProgress, QString::null, 30));
 
 	DEBUGKPILOT << fname << ": doing dlp_ReadSysInfo..." << endl;
 
 	struct SysInfo sys_info;
 	if (dlp_ReadSysInfo(fPilotSocket, &sys_info) < 0)
 	{
-		QApplication::postEvent(link(),
-			new DeviceCommEvent(EventLogError,
-			i18n("Unable to read system information from Pilot")));
+		QApplication::postEvent(link(), new DeviceCommEvent(EventLogError,
+				i18n("Unable to read system information from Pilot")));
 
 		link()->fLinkStatus=PilotLinkError;
-		return false;
+		reset();
+		return;
 	}
 	else
 	{
 		DEBUGKPILOT << fname << ": dlp_ReadSysInfo successful..." << endl;
 
-        	KPILOT_DELETE(link()->fPilotSysInfo);
+		KPILOT_DELETE(link()->fPilotSysInfo);
 		link()->fPilotSysInfo = new KPilotSysInfo(&sys_info);
 		DEBUGKPILOT << fname
 			<< ": RomVersion: [" << link()->fPilotSysInfo->getRomVersion()
@@ -534,10 +457,13 @@ bool DeviceCommThread::open(const QString &device)
 			<< "]" << endl;
 	}
 
-	QApplication::postEvent(link(),
-		new DeviceCommEvent(EventLogProgress, QString::null, 60));
+	// If we've made it this far, make sure our USB workaround timer doesn't fire!
+	fWorkaroundUSBTimer->stop();
+	KPILOT_DELETE(fWorkaroundUSBTimer);
 
-        KPILOT_DELETE(link()->fPilotUser);
+	QApplication::postEvent(link(), new DeviceCommEvent(EventLogProgress, QString::null, 60));
+
+	KPILOT_DELETE(link()->fPilotUser);
 	link()->fPilotUser = new KPilotUser;
 
 	DEBUGKPILOT << fname << ": doing dlp_ReadUserInfo..." << endl;
@@ -549,8 +475,7 @@ bool DeviceCommThread::open(const QString &device)
 	DEBUGKPILOT << fname
 		<< ": Read user name: [" << n << "]" << endl;
 
-	QApplication::postEvent(link(),
-		new DeviceCommEvent(EventLogProgress, i18n("Checking last PC..."), 90));
+	QApplication::postEvent(link(), new DeviceCommEvent(EventLogProgress, i18n("Checking last PC..."), 90));
 
 	/* Tell user (via Pilot) that we are starting things up */
 	if ((ret=dlp_OpenConduit(fPilotSocket)) < 0)
@@ -558,79 +483,36 @@ bool DeviceCommThread::open(const QString &device)
 		DEBUGKPILOT << fname
 			<< ": dlp_OpenConduit returned: [" << ret << "]" << endl;
 
-		QApplication::postEvent(link(),
-			new DeviceCommEvent(EventLogError,
-			i18n("Could not read user information from the Pilot. "
-			"Perhaps you have a password set on the device?")));
+		QApplication::postEvent(link(), new DeviceCommEvent(EventLogError,
+				i18n("Could not read user information from the Pilot. "
+						"Perhaps you have a password set on the device?")));
 
 	}
 	link()->fLinkStatus = AcceptedDevice;
 
-	QApplication::postEvent(link(),
-		new DeviceCommEvent(EventLogProgress, QString::null, 100));
+	QApplication::postEvent(link(), new DeviceCommEvent(EventLogProgress, QString::null, 100));
+
+	DeviceCommEvent * ev = new DeviceCommEvent(EventDeviceReady);
+	ev->setCurrentSocket(fPilotSocket);
+	QApplication::postEvent(link(), ev);
 
-	return true;
 }
 
-void DeviceCommThread::close()
+void DeviceCommThread::workaroundUSB()
 {
 	FUNCTIONSETUP;
 
-	pi_close(fTempSocket);
-
-	if (fPilotSocket != -1)
-	{
-		DEBUGKPILOT << fname
-			<< ": device comm thread closing socket: ["
-			<< fPilotSocket << "]" << endl;
-
-		pi_close(fPilotSocket);
-	}
-	fPilotSocket = (-1);
-
-	DeviceMap::self()->unbindDevice( link()->fRealPilotPath );
+	reset();
 }
 
-
 void DeviceCommThread::run()
 {
 	FUNCTIONSETUP;
-	int sleepBetweenPoll = SecondsBetweenPoll;
 	fDone = false;
 
-	DEBUGKPILOT << fname << ": Polling every: ["
-		<< sleepBetweenPoll << "] seconds." << endl;
-
-	while (!fDone)
-	{
-		openDevice();
-
-		if (link()->fLinkStatus == AcceptedDevice)
-		{
-			DeviceCommEvent * ev = new DeviceCommEvent(EventDeviceReady);
-			ev->setCurrentSocket(fPilotSocket);
-			QApplication::postEvent(link(), ev);
-
-			break;
-		}
-		else
-		{
-			if (link()->fMessages->shouldPrint(Messages::OpenFailMessage))
-			{
-				QApplication::postEvent(link(), 
-					new DeviceCommEvent(EventLogMessage, 
-						i18n("Could not open device: %1 (will retry)")
-						.arg(link()->fPilotPath)));
-			}
-			close();
-
-			// sleep before trying again
-			QThread::sleep(sleepBetweenPoll);
-		}
-	}
-
-	DEBUGKPILOT << fname << ": comm thread waiting to be done..." << endl;
+	startOpenTimer(this, fOpenTimer);
 
+	int sleepBetweenPoll = 2;
 	// keep the thread alive until we're supposed to be done
 	while (!fDone)
 	{
@@ -645,17 +527,10 @@ void DeviceCommThread::run()
 	DEBUGKPILOT << fname << ": comm thread now done..." << endl;
 }
 
-
-
-
-KPilotDeviceLink::KPilotDeviceLink(QObject * parent, const char *name, const QString &tempDevice) :
-	KPilotLink(parent, name),
-	fLinkStatus(Init),
-	fWorkaroundUSB(false),
-	fPilotSocket(-1),
-	fTempDevice(tempDevice),
-	fMessages(new Messages(this)),
-	fDeviceCommThread(0L)
+KPilotDeviceLink::KPilotDeviceLink(QObject * parent, const char *name,
+		const QString &tempDevice) :
+	KPilotLink(parent, name), fLinkStatus(Init), fWorkaroundUSB(false),
+			fPilotSocket(-1), fTempDevice(tempDevice), fMessages(new Messages(this)), fDeviceCommThread(0L)
 {
 	FUNCTIONSETUP;
 
@@ -673,72 +548,48 @@ KPilotDeviceLink::~KPilotDeviceLink()
 	KPILOT_DELETE(fMessages);
 }
 
-/* virtual */ bool KPilotDeviceLink::isConnected() const
+/* virtual */bool KPilotDeviceLink::isConnected() const
 {
-	 return fLinkStatus == AcceptedDevice;
+	return fLinkStatus == AcceptedDevice;
 }
 
-void KPilotDeviceLink::customEvent(QCustomEvent *e)
+/* virtual */bool KPilotDeviceLink::event(QEvent *e)
 {
 	FUNCTIONSETUP;
 
+	bool handled = false;
+
 	if ((int)e->type() == EventDeviceReady)
 	{
-		DeviceCommEvent* t = dynamic_cast<DeviceCommEvent*>(e);
-		if (t)
-		{
-			fPilotSocket = t->currentSocket();
-			emit deviceReady( this );
-		}
-		else
-		{
-			DEBUGKPILOT << fname
-				<< ": unable to cast event: [" << e << "]" << endl;
-		}
+		DeviceCommEvent* t = static_cast<DeviceCommEvent*>(e);
+		fPilotSocket = t->currentSocket();
+		emit deviceReady( this);
+		handled = true;
 	}
 	else if ((int)e->type() == EventLogMessage)
 	{
-		DeviceCommEvent* t = dynamic_cast<DeviceCommEvent*>(e);
-		if (t)
-		{
-			emit logMessage(t->message());
-		}
-		else
-		{
-			DEBUGKPILOT << fname
-				<< ": unable to cast event: [" << e << "]" << endl;
-		}
+		DeviceCommEvent* t = static_cast<DeviceCommEvent*>(e);
+		emit logMessage(t->message());
+		handled = true;
 	}
 	else if ((int)e->type() == EventLogError)
 	{
-		DeviceCommEvent* t = dynamic_cast<DeviceCommEvent*>(e);
-		if (t)
-		{
-			emit logError(t->message());
-		}
-		else
-		{
-			DEBUGKPILOT << fname
-				<< ": unable to cast event: [" << e << "]" << endl;
-		}
+		DeviceCommEvent* t = static_cast<DeviceCommEvent*>(e);
+		emit logError(t->message());
+		handled = true;
 	}
 	else if ((int)e->type() == EventLogProgress)
 	{
-		DeviceCommEvent* t = dynamic_cast<DeviceCommEvent*>(e);
-		if (t)
-		{
-			emit logProgress(t->message(), t->progress());
-		}
-		else
-		{
-			DEBUGKPILOT << fname
-				<< ": unable to cast event: [" << e << "]" << endl;
-		}
+		DeviceCommEvent* t = static_cast<DeviceCommEvent*>(e);
+		emit logProgress(t->message(), t->progress());
+		handled = true;
 	}
 	else
 	{
-		KPilotLink::customEvent(e);
+		handled = KPilotLink::event(e);
 	}
+
+	return handled;
 }
 
 void KPilotDeviceLink::stopCommThread()
@@ -750,12 +601,12 @@ void KPilotDeviceLink::stopCommThread()
 
 		// try to wait for our thread to finish, but don't
 		// block the main thread forever
-		if (fDeviceCommThread->running()) 
+		if (fDeviceCommThread->running())
 		{
 			DEBUGKPILOT << fname
 				<< ": comm thread still running. "
 				<< "waiting for it to complete." << endl;
-			bool done = fDeviceCommThread->wait(30000);
+			bool done = fDeviceCommThread->wait(5000);
 			if (!done)
 			{
 				DEBUGKPILOT << fname
@@ -798,7 +649,7 @@ void KPilotDeviceLink::reset(const QString & dP)
 
 	fPilotPath = dP;
 	if (fPilotPath.isEmpty())
-	   fPilotPath = fTempDevice;
+		fPilotPath = fTempDevice;
 	if (fPilotPath.isEmpty())
 		return;
 
@@ -819,9 +670,8 @@ void KPilotDeviceLink::startCommThread()
 		QString msg = i18n("The Pilot device is not configured yet.");
 		WARNINGKPILOT << msg << endl;
 
-
 		fLinkStatus = PilotLinkError;
-		
+
 		emit logError(msg);
 		return;
 	}
@@ -830,7 +680,6 @@ void KPilotDeviceLink::startCommThread()
 	fDeviceCommThread->start();
 }
 
-
 void KPilotDeviceLink::reset()
 {
 	FUNCTIONSETUP;
@@ -858,7 +707,7 @@ void KPilotDeviceLink::checkDevice()
 		if (!(fi.isReadable() && fi.isWritable()))
 		{
 			emit logError(i18n("Pilot device %1 is not read-write.")
-				.arg(fPilotPath));
+			.arg(fPilotPath));
 		}
 	}
 	else
@@ -866,32 +715,33 @@ void KPilotDeviceLink::checkDevice()
 		// It doesn't exist, mention this in the log
 		// (relevant as long as we use only one device type)
 		//
-		emit logError(i18n("Pilot device %1 does not exist. "
-			"Probably it is a USB device and will appear during a HotSync.")
+		emit
+				logError(i18n("Pilot device %1 does not exist. "
+						"Probably it is a USB device and will appear during a HotSync.")
 				.arg(fPilotPath));
 		// Suppress all normal and error messages about opening the device.
-		fMessages->block(Messages::OpenMessage | Messages::OpenFailMessage, true);
+		fMessages->block(Messages::OpenMessage | Messages::OpenFailMessage,
+				true);
 	}
 }
 
-void KPilotDeviceLink::setTempDevice( const QString &d )
+void KPilotDeviceLink::setTempDevice(const QString &d)
 {
 	fTempDevice = d;
-	DeviceMap::self()->bindDevice( fTempDevice );
+	DeviceMap::self()->bindDevice(fTempDevice);
 }
 
-
-/* virtual */ bool KPilotDeviceLink::tickle()
+/* virtual */bool KPilotDeviceLink::tickle()
 {
 	// No FUNCTIONSETUP here because it may be called from
 	// a separate thread.
 	return pi_tickle(pilotSocket()) >= 0;
 }
 
-/* virtual */ void KPilotDeviceLink::addSyncLogEntryImpl( const QString &entry )
+/* virtual */void KPilotDeviceLink::addSyncLogEntryImpl(const QString &entry)
 {
 	dlp_AddSyncLogEntry(fPilotSocket,
-		const_cast<char *>((const char *)Pilot::toPilot(entry)));
+			const_cast<char *>((const char *)Pilot::toPilot(entry)));
 }
 
 bool KPilotDeviceLink::installFile(const QString & f, const bool deleteFile)
@@ -904,17 +754,16 @@ bool KPilotDeviceLink::installFile(const QString & f, const bool deleteFile)
 		return false;
 
 	char buffer[PATH_MAX];
-	memset(buffer,0,PATH_MAX);
-	strlcpy(buffer,QFile::encodeName(f),PATH_MAX);
-	struct pi_file *pf =
-		pi_file_open(buffer);
+	memset(buffer, 0, PATH_MAX);
+	strlcpy(buffer, QFile::encodeName(f), PATH_MAX);
+	struct pi_file *pf = pi_file_open(buffer);
 
 	if (!f)
 	{
 		WARNINGKPILOT << "Cannot open file " << f << endl;
 		emit logError(i18n
 			("<qt>Cannot install the file &quot;%1&quot;.</qt>").
-			arg(f));
+		arg(f));
 		return false;
 	}
 
@@ -922,18 +771,18 @@ bool KPilotDeviceLink::installFile(const QString & f, const bool deleteFile)
 	{
 		WARNINGKPILOT << "Cannot pi_file_install " << f << endl;
 		emit logError(i18n
-			("<qt>Cannot install the file &quot;%1&quot;.</qt>").
-			arg(f));
+				("<qt>Cannot install the file &quot;%1&quot;.</qt>").
+		arg(f));
 		return false;
 	}
 
 	pi_file_close(pf);
-	if (deleteFile) QFile::remove(f);
+	if (deleteFile)
+		QFile::remove(f);
 
 	return true;
 }
 
-
 int KPilotDeviceLink::openConduit()
 {
 	return dlp_OpenConduit(fPilotSocket);
@@ -941,7 +790,7 @@ int KPilotDeviceLink::openConduit()
 
 QString KPilotDeviceLink::statusString(LinkStatus l)
 {
-	QString s = CSL1("KPilotDeviceLink=");
+	QString s= CSL1("KPilotDeviceLink=");
 
 	switch (l)
 	{
@@ -979,21 +828,21 @@ QString KPilotDeviceLink::statusString(LinkStatus l)
 
 QString KPilotDeviceLink::statusString() const
 {
-	return statusString( status() );
+	return statusString(status() );
 }
 
-void KPilotDeviceLink::endSync( EndOfSyncFlags f )
+void KPilotDeviceLink::endSync(EndOfSyncFlags f)
 {
 	FUNCTIONSETUP;
 
-	if ( UpdateUserInfo == f )
+	if (UpdateUserInfo == f)
 	{
 		getPilotUser().setLastSyncPC((unsigned long) gethostid());
 		getPilotUser().setLastSyncDate(time(0));
 
 		DEBUGKPILOT << fname << ": Writing username " << getPilotUser().name() << endl;
 
-		dlp_WriteUserInfo(pilotSocket(),getPilotUser().data());
+		dlp_WriteUserInfo(pilotSocket(), getPilotUser().data());
 		addSyncLogEntry(i18n("End of HotSync\n"));
 	}
 	dlp_EndOfSync(pilotSocket(), 0);
@@ -1001,34 +850,33 @@ void KPilotDeviceLink::endSync( EndOfSyncFlags f )
 	KPILOT_DELETE(fPilotUser);
 }
 
-
-int KPilotDeviceLink::getNextDatabase(int index,struct DBInfo *dbinfo)
+int KPilotDeviceLink::getNextDatabase(int index, struct DBInfo *dbinfo)
 {
 	FUNCTIONSETUP;
 
-	pi_buffer_t buf = { 0,0,0 };
-	int r = dlp_ReadDBList(pilotSocket(),0,dlpDBListRAM,index,&buf);
+	pi_buffer_t buf = 	{ 0, 0, 0 };
+	int r = dlp_ReadDBList(pilotSocket(), 0, dlpDBListRAM, index, &buf);
 	if (r >= 0)
 	{
-		memcpy(dbinfo,buf.data,sizeof(struct DBInfo));
+		memcpy(dbinfo, buf.data, sizeof(struct DBInfo));
 	}
 	return r;
 }
 
 // Find a database with the given name. Info about the DB is stored into dbinfo (e.g. to be used later on with retrieveDatabase).
 int KPilotDeviceLink::findDatabase(const char *name, struct DBInfo *dbinfo,
-	int index, unsigned long type, unsigned long creator)
+		int index, unsigned long type, unsigned long creator)
 {
 	FUNCTIONSETUP;
-	return dlp_FindDBInfo(pilotSocket(), 0, index,
-		const_cast<char *>(name), type, creator, dbinfo);
+	return dlp_FindDBInfo(pilotSocket(), 0, index, const_cast<char *>(name),
+			type, creator, dbinfo);
 }
 
-bool KPilotDeviceLink::retrieveDatabase(const QString &fullBackupName,	DBInfo *info)
+bool KPilotDeviceLink::retrieveDatabase(const QString &fullBackupName,
+		DBInfo *info)
 {
 	FUNCTIONSETUP;
 
-
 	if (fullBackupName.isEmpty() || !info)
 	{
 		// Don't even bother trying to convert or retrieve.
@@ -1039,7 +887,7 @@ bool KPilotDeviceLink::retrieveDatabase(const QString &fullBackupName,	DBInfo *i
 		<< " to " << fullBackupName << endl;
 
 	QCString encodedName = QFile::encodeName(fullBackupName);
-	struct pi_file *f = pi_file_create(encodedName,info);
+	struct pi_file *f = pi_file_create(encodedName, info);
 
 	if (!f)
 	{
@@ -1059,7 +907,6 @@ bool KPilotDeviceLink::retrieveDatabase(const QString &fullBackupName,	DBInfo *i
 	return true;
 }
 
-
 KPilotLink::DBInfoList KPilotDeviceLink::getDBList(int cardno, int flags)
 {
 	bool cont=true;
@@ -1067,10 +914,11 @@ KPilotLink::DBInfoList KPilotDeviceLink::getDBList(int cardno, int flags)
 	int index=0;
 	while (cont)
 	{
-		pi_buffer_t buf = { 0,0,0 };
+		pi_buffer_t buf = { 0, 0, 0 };
 		pi_buffer_clear(&buf);
 		// DBInfo*dbi=new DBInfo();
-		if (dlp_ReadDBList(pilotSocket(), cardno, flags | dlpDBListMultiple, index, &buf)<0)
+		if (dlp_ReadDBList(pilotSocket(), cardno, flags | dlpDBListMultiple,
+				index, &buf)<0)
 		{
 			cont=false;
 		}
@@ -1080,9 +928,9 @@ KPilotLink::DBInfoList KPilotDeviceLink::getDBList(int cardno, int flags)
 			DBInfo *db_it = (DBInfo *)buf.data;
 			int info_count = buf.used / sizeof(struct DBInfo);
 
-			while(info_count>0)
+			while (info_count>0)
 			{
-				memcpy(&db_n,db_it,sizeof(struct DBInfo));
+				memcpy(&db_n, db_it, sizeof(struct DBInfo));
 				++db_it;
 				info_count--;
 				dbs.append(db_n);
@@ -1098,21 +946,20 @@ const KPilotCard *KPilotDeviceLink::getCardInfo(int card)
 	KPilotCard *cardinfo=new KPilotCard();
 	if (dlp_ReadStorageInfo(pilotSocket(), card, cardinfo->cardInfo())<0)
 	{
-		WARNINGKPILOT << "Could not get info for card "
-			<< card << endl;
+		WARNINGKPILOT << "Could not get info for card " << card << endl;
 
 		KPILOT_DELETE(cardinfo);
 		return 0L;
-	};
+	}
 	return cardinfo;
 }
 
-PilotDatabase *KPilotDeviceLink::database( const QString &name )
+PilotDatabase *KPilotDeviceLink::database(const QString &name)
 {
 	return new PilotSerialDatabase( this, name );
 }
 
-PilotDatabase *KPilotDeviceLink::database( const DBInfo *info )
+PilotDatabase *KPilotDeviceLink::database(const DBInfo *info)
 {
 	return new PilotSerialDatabase( this, info );
 }
