@@ -314,6 +314,15 @@ icalvalue* icalvalue_new_from_string_with_error(icalvalue_kind kind,const char*
 	}
 
     case ICAL_BINARY_VALUE:
+    {
+        icalattach *attach;
+        attach = icalattach_new_from_data (str, 0, 0);
+        if ( !attach )
+          break;
+        value = icalvalue_new_attach (attach);
+        icalattach_unref (attach);
+        break;
+    }
     case ICAL_BOOLEAN_VALUE:
         {
             /* HACK */
@@ -801,8 +810,14 @@ icalvalue_attach_as_ical_string(const icalvalue* value)
 	str = icalmemory_tmp_buffer (strlen (url) + 1);
 	strcpy (str, url);
 	return str;
-    } else
-	return icalvalue_binary_as_ical_string (value);
+    } else {
+/* 	return icalvalue_binary_as_ical_string (value);*/
+      const char *data = 0;
+      data = (const char*)icalattach_get_data(a);
+      str = icalmemory_tmp_buffer (strlen (data) + 1);
+      strcpy (str, data);
+      return str;
+}
 }
 
 
